# -*- coding: iso_8859_1 -*-

import numpy as np
import matplotlib.pyplot as plt

from matplotlib import cm
from matplotlib.widgets import Slider
from aspylib.astro import stats


def display(data, title='', show=True):
	"""
	---------------------
	Purpose
	Displays the data of a FITS image.
	The display window allows to zoom in, zoom out, adjust visualisation thresholds, and save the displayed image in an image file.
	---------------------
	Inputs
	* data (2D Numpy array) = the data of one FITS image, as obtained for instance by reading a list containing a single FITS file with the function astro.get_imagedata(), or by reading a list of several images with astro.get_imagedata() and then extracting the data of a single image from the returned data.
	The function astro.display() works with any 2D Numpy array.
	* title (string) = if specified, defines the title of the window where the image is displayed.
	* show (boolean) = default value is True. If True, then the image is displayed. If False, the image is not displayed so the user can add extra features in the matplotlib window, before displaying the image with matplotlib.pyplot.show().
	---------------------
	Outputs = fig, ax
	* fig (matplotlib.figure.Figure instance) = the figure that contains the displayed image
	* ax (matplotlib.axes.Axes instance) = the figure element that corresponds to the displayed image (other figure elements are the sliders)
	IMPORTANT = If show=False the function returns fig and ax. If show=True the function returns no output.
	---------------------
	"""
	data = np.array(data,dtype=np.int)
	med = stats.get_median(data)[0]
	dsig = np.std(data.flatten())
	
	fig = plt.figure()
	ax = plt.subplot(111)
	plt.subplots_adjust(bottom=0.2)
	
	h = plt.imshow(data, interpolation='nearest',cmap=cm.gray,origin='lower',vmin = med-0.5*dsig, vmax =med+2*dsig, aspect='equal', extent=None)
	
	#------ defining sliders ------
	axmini = plt.axes([0.1, 0.05, 0.8, 0.03])
	axmaxi  = plt.axes([0.1, 0.1, 0.8, 0.03])

	smini = Slider(axmini, 'min', med-5*dsig, med+20*dsig, dragging=False, valinit=med-0.5*dsig, valfmt='%1.0f')
	smaxi = Slider(axmaxi, 'max', med-5*dsig, med+20*dsig, dragging=False, valinit=med+2*dsig, valfmt='%1.0f')

	smini.poly.set_color('0.5')
	smaxi.poly.set_color('0.5')
	def update(val):
		mini = smini.val
		maxi = smaxi.val
		if mini<=maxi:
			dat = data
			h.set_data(dat)
			h.set_clim(vmin=smini.val, vmax=smaxi.val)
		else:
			dat = maxi+mini-data
			h.set_data(dat)
			h.set_clim(vmin=smaxi.val, vmax=smini.val)
			
		plt.draw()
	
	smini.on_changed(update)
	smaxi.on_changed(update)

	if title=='':
		fig.canvas.set_window_title('Display')
	else:
		fig.canvas.set_window_title('Display: ' + title)
	
	if show:
		plt.show()
	else:
		return fig, ax


def select(data, title='', show=True, nb=-1, color='none', facecolor='none', edgecolor='none', marker='o', size=50):
	"""
	---------------------
	Purpose
	Displays the data of a FITS image, and returns the (x,y) coordinates of selected pixels on the displayed FITS image.
	The display window allows to zoom in, zoom out, adjust visualisation thresholds, and save the displayed image in an image file.
	The selection is made by placing the cursor on a specific pixel, and then making a mouse right-click.
	---------------------
	Inputs
	* data (2D Numpy array) = the data of one FITS image, as obtained for instance by reading a list containing a single FITS file with the function astro.get_imagedata(), or by reading a list of several images with astro.get_imagedata() and then extracting the data of a single image from the returned data.
	The function astro.display() works with any 2D Numpy array.
	* title (string) = if specified, defines the title of the window where the image is displayed.
	* show (boolean) = default value is True. If True, then the image is displayed. If False, the image is not displayed so the user can add extra features in the matplotlib window, before displaying the image with matplotlib.pyplot.show().
	* nb (integer) = the number of pixels to be selected. If equal to -1, an arbitrary number of pixels can be selected: the selection is finished with a mouse right-click outside of the displayed image.
	* color (string) = optional argument, if set to a Matplotlib color then a number with color=color is displayed on the picture next to each selected pixel.
	* facecolor (string) = optional argument, if set to a Matplotlib color then a Marker with facecolor = facecolor is placed on the image at each selected pixel.
	* edgecolor (string) = optional argument, if set to a Matplotlib color then a Marker  with edgecolor = edgecolor is placed on the image at each selected pixel.
	* marker (string) = optional argument, gives the shape of the marker.
	* size (integer) = optional argument, gives the size of the marker.
	The inputs "color", "marker" and "size" can take any value that is accepted by the Matplotlib.pyplot.scatter() function for the arguments (respectively): c, marker, s. For more information see the Matplotlib help.
	---------------------
	Outputs = xy, fig, ax
	* xy (list) = has the form [[x1,y1],[x2,y2],...,[xn,yn]]. The list [xi,yi] gives the coordinates of the ith selected pixel.
	* fig (matplotlib.figure.Figure instance) = the figure that contains the displayed image
	* ax (matplotlib.axes.Axes instance) = the figure element that corresponds to the displayed image (other figure elements are the sliders)
	IMPORTANT = If show=False, the function returns xy, fig, ax. If show=True the function returns only xy.
	---------------------
	"""

	print("")
	print("aspylib.astro.select():")
		
	#displaying the data
	data = np.array(data,dtype=np.int)
	med = stats.get_median(data)[0]
	dsig = np.std(data.flatten())

	fig = plt.figure()
	ax = plt.subplot(111)
	plt.subplots_adjust(bottom=0.2)
	
	h = plt.imshow(data, interpolation='nearest',cmap=cm.gray,origin='lower',vmin = med-0.5*dsig, vmax =med+2*dsig, aspect='equal', extent=None)
	
	if title=='':
		fig.canvas.set_window_title('Select')
	else:
		fig.canvas.set_window_title('Select: ' + title)
	
	#defining sliders
	axmini = plt.axes([0.1, 0.05, 0.8, 0.03])
	axmaxi  = plt.axes([0.1, 0.1, 0.8, 0.03])

	smini = Slider(axmini, 'min', med-5*dsig, med+20*dsig, dragging=False, valinit=med-0.5*dsig, valfmt='%1.0f')
	smaxi = Slider(axmaxi, 'max', med-5*dsig, med+20*dsig, dragging=False, valinit=med+2*dsig, valfmt='%1.0f')

	smini.poly.set_color('0.5')
	smaxi.poly.set_color('0.5')

	#variables for star selection
	xy = []
	cid = 0 
	global i
	i=0 

	def update(val):
		mini = smini.val
		maxi = smaxi.val
		if mini<=maxi:
			dat = data
			h.set_data(dat)
			h.set_clim(vmin=smini.val, vmax=smaxi.val)
		else:
			dat = maxi+mini-data
			h.set_data(dat)
			h.set_clim(vmin=smaxi.val, vmax=smini.val)
			
		plt.draw()
	
	def on_click(event): 
		global i 
		if event.inaxes in [ax]: 
			if event.button==3: 
				xy.append([int(round(event.ydata)),int(round(event.xdata))]) 
				print('vert=',xy[-1][0],'horiz=',xy[-1][1])
				i = i+1 
				ax.scatter(xy[-1][1],xy[-1][0],s=size, marker=marker, facecolor=facecolor, edgecolor=edgecolor) 
				ax.annotate(str(i), xy=(xy[-1][1]+10, xy[-1][0]+10),  xycoords='data', color=color) 
				plt.draw()
				if len(xy)==nb: 
					fig.canvas.mpl_disconnect(cid) 
		if event.inaxes is None: 
			if nb<0: 
				if event.button==3: 
					fig.canvas.mpl_disconnect(cid) 
				
	smini.on_changed(update)
	smaxi.on_changed(update)

	cid = fig.canvas.mpl_connect('button_press_event', on_click) 
	if show:
		plt.show()
		return xy
	else:
		return xy, fig, ax
