# -*- coding: iso_8859_1 -*-

import numpy as np
import sys
import scipy.fftpack as fft

from aspylib.astro import process2, process3, transform

def find_stars_accurate(image, nb_stars, fit_area, excluded = [], included = [], saturation = [False,0], text_display = True, filter = 0):
	"""
	---------------------
	Purpose
	Takes an image and returns the accurate positions of the N brightest stars, N being specified by the user.
	The accurate star centroids are found by fitting the star with an elliptical gaussian PSF.
	---------------------
	Inputs
	* image (2D Numpy array) = the data of one FITS image, as obtained for instance by reading one FITS image file with the function astro.get_imagedata().
	* nb_stars (integer) = the number of stars requested.
	* fit_area (integer, must be >0) = defines the +/- size of the square window used for star fitting (the square window has a size 2*fit_area+1).
	The window size must be large enough to include the stars for the PSF fit, but small enough not to include contributions from other stars.
	* excluded (list) = optional argument. List of 4 elements lists, with the form [[x11,x12,y11,y12], ..., [xn1,xn2,yn1,yn2]]. Each element is a small part of the image that will be excluded for the star detection.
	This optional parameter allows to reject problematic areas in the image (e.g. saturated star(s), artefact(s), moving object(s)..)
	* included (list) = optional argument. List of 4 integers, with the form [x1,x2,y1,y2]. It defines a subpart of the image where all the stars will be detected.
	Both excluded and included can be used simultaneously.
	* saturation (list) = optional argument. Two elements lists: if saturation[0] is True, the objects with maximum signal >= saturation[1] are disregarded.
	* text_display (boolean) = optional argument. If True then some information about the star detection is printed in the console.
	* filter (integer) = optional argument. Defines if and how inaccurate stars positions are removed. Default value is 0, in which case no positions are removed. If equal to 1 then np.nan positions are removed; if equal to 2, np.nan or integer coordinates (such that np.round(x)==x) are removed.
	If some positions are removed, the final number of stars returned by the function becomes smaller than nb_stars.
	---------------------
	Output (2D Numpy array) = Numpy array with the form [[x1,y1],[x2,y2],...,[xn,yn]], that contains the approximate x and y positions for each star.
	---------------------
	"""
	stars = process2.find_stars(image, nb_stars, excluded = excluded, included = included, detection_area = 2, saturation = saturation, margin = 5, text_display = text_display)
	
	stars2 = process3.fit_stars(image, stars, fit_area, maxshift = 2, error_messages = 1, full_info=False)
	stars3 = process3.remove_inaccurate_positions(stars2, filter)
	
	if len(stars3)<nb_stars:
		print("")
		print("aspylib.astro.find_stars_accurate():")
		print(nb_stars, "stars requested but only", len(stars3), "returned")
	return stars3

def find_shift_stars_matching(stars1, stars2, text_display=False):
	"""
	---------------------
	Purpose
	This function calculates the accurate (non-integer) X and Y shifts between two images, by matching the positions of the provided stars.
	Accurate star positions (i.e. non-integer, obtained with a PSF fit) must be provided as inputs.
	---------------------
	Inputs
	* stars1 (2D Numpy array) = array with the form [[x1,y1],[x2,y2],...,[xn,yn]], that contains the approximate positions of some stars on image1.
	* stars2 (2D Numpy array) = array containing the approximate positions of some stars on image2 (not necessarily the same stars and not necessarily the same number of stars).
	* text_display (boolean) = optional argument. If equal to True, some information is written in the console window. If equal to False (default) nothing is written.
	---------------------
	Output (list) = two elements list with the form [dx,dy], that contains the accurate X and Y shifts of image2 with respect to image1. The X axis is vertical, the Y axis horizontal.
	---------------------
	"""

	step = 1.			#shift difference value below which shifts are assumed to belong to the same translation scenario
	#-------------------------------------------------------------------------------------
	#	finding stars
	#-------------------------------------------------------------------------------------
	
	if (stars1.shape[0]==0)or(stars2.shape[0]==0):
		print("")
		print("aspylib.astro.find_shift_stars_matching():")
		if (sys.version[0]=="2"):
			raw_input('no stars could be detected, shift will not be calculated.\nPress Enter to close')
		elif (sys.version[0]=="3"):
			input('no stars could be detected, shift will not be calculated.\nPress Enter to close')
		sys.exit()		

	#-------------------------------------------------------------------------------------
	#	looking for x and y shift
	#-------------------------------------------------------------------------------------
	r1 = range(len(stars1))
	r2 = range(len(stars2))

	delta_x = [[stars2[i,0]-stars1[j,0] for i in r2] for j in r1]
	delta_y = [[stars2[i,1]-stars1[j,1] for i in r2] for j in r1]

	delta_x = np.array(delta_x).flatten()
	delta_y = np.array(delta_y).flatten()
	#-------------------------------------------------------------------------------------
	rx = range(len(delta_x))
	ry = range(len(delta_y))
	
	nb_x = [[delta_x[i] - delta_x[j] for i in rx] for j in rx] 
	nb_y = [[delta_y[i] - delta_y[j] for i in ry] for j in ry] 

	nb_x = np.array(nb_x)
	nb_y = np.array(nb_x)
	#-------------------------------------------------------------------------------------
	nb = (np.abs(nb_x)<=step) * (np.abs(nb_y)<=step)
	nb = np.sum(nb,axis=1)
	nb_max = np.max(nb)

	if text_display:
		print("")
		print("aspylib.astro.find_shift_stars_matching():")
		print('shift comparison result =')
		print(nb)

	#if list(nb).count(nb_max) != nb_max:
	#	if (sys.version[0]=="2"):
	#		raw_input('more than one solution, shift could not be calculated.\nPress Enter to close')
	#	elif (sys.version[0]=="3"):
	#		input('more than one solution, shift could not be calculated.\nPress Enter to close')
	#	sys.exit()
	
	if nb_max>10:
		dx = [delta_x[i] for i in range(len(delta_x)) if nb[i]>=nb_max-1]
		dy = [delta_y[i] for i in range(len(delta_y)) if nb[i]>=nb_max-1]
	else:
		dx = [delta_x[i] for i in range(len(delta_x)) if nb[i]==nb_max]
		dy = [delta_y[i] for i in range(len(delta_y)) if nb[i]==nb_max]
	
	return [np.ma.median(dx), np.ma.median(dy)]
	
def find_shift_crosscorrelation(image1, image2):
	"""
	---------------------
	Purpose
	This function calculates the accurate (non-integer) X and Y shifts between two images, by calculating their cross-correlation.
	The method may fail if there are too bright moving objects, or in the presence of significant artefacts (dead pixels, strong cosmic ray impacts, etc).
	---------------------
	Inputs
	* image1 (2D Numpy array) = the data of the first FITS image, as obtained for instance by reading one FITS image file with the function astro.get_imagedata().
	* image2 (2D Numpy array) = the data of the second FITS image.
	---------------------
	Output (list) = two elements list with the form [dx,dy], that contains the accurate X and Y shifts of image2 with respect to image1. The X axis is vertical, the Y axis horizontal.
	---------------------
	"""
	
	#--- calculates cross-correlation between two images ---
	im1 = fft.fft2(image1)
	im2 = fft.fft2(image2)
	crosscorr = fft.ifft2(im1*np.ma.conjugate(im2))

	#--- finds the position of the max of the modulus of the cross-correlation ---
	modulus = fft.fftshift(np.abs(crosscorr))
	dx, dy = np.where(modulus==np.max(modulus.flatten()))
	#needed due to different behaviour of np.where depending on whether there is 1 or more positions in the image verifying the condition
	if len(dx)==1:
		dx=dx[0]
		dy=dy[0]
	else:
		dy=dx[1]
		dx=dx[0]

	#--- zoom in on max position with scaling factor "size", finds again the position of the max with better accuracy ---
	size = 100

	#--- nouvelle instruction ---
	submod = np.zeros((201,201))
	dim1,dim2 = np.shape(modulus)
	submod[max(size-dx,0):min(201+dim1-dx-size-1,201),max(size-dy,0):min(201+dim2-dy-size-1,201)] = modulus[max(dx-size,0):min(dx+size+1,dim1),max(dy-size,0):min(dy+size+1,dim2)]
	#print [dx-size,dx+size+1,dy-size,dy+size+1]
	#print [max(dx-size,0),min(dx+size+1,dim1),max(dy-size,0),min(dy+size+1,dim2)]
	#print [max(size-dx,0),min(201+dim1-dx-size-1,201),max(size-dy,0),min(201+dim2-dy-size-1,201)]
	#----------------------------
	
	#--- ancienne instruction ---
	#submod = modulus[dx-size:dx+size+1,dy-size:dy+size+1]
	#----------------------------
	
	submod2 = transform.scale_Bspline(submod, [size, size], 100., 3)
	dx2, dy2 = np.where(submod2==np.max(submod2.flatten()))
	#needed due to different behaviour of np.where depending on whether there is 1 or more positions in the image verifying the condition
	if len(dx2)==1:
		dx2=dx2[0]
		dy2=dy2[0]
	else:
		dy2=dx2[1]
		dx2=dx2[0]

	#--- combines previous results
	dx3 = image1.shape[0]/2. - np.float(dx+(dx2-100.)/100.)
	dy3 = image1.shape[1]/2. - np.float(dy+(dy2-100.)/100.)
	return [dx3, dy3]

