# -*- coding: iso_8859_1 -*-

import astropy.io.fits
import os, datetime, sys
import numpy as np

def save_imagelist(images, headers, Imagelist, bitpix=16, overwrite=True):
	"""
	---------------------
	Purpose
	Saves a list of FITS images, given their data and header information.
	The data can be saved under unsigned 16bits integer format (BITPIX=16, BSCALE=1, BZERO=32768) with values in the range [0-65535] ; or as 32bits IEEE float format.
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = array that contains the data of one image (2D case) or the data of a list of images (3D case).
	* headers (list) = has the form [dictionary_1, dictionary_2, ..., dictionary_n]. Each list element dictionary_i is a Python dictionary that contains all the headers for the ith FITS image.
	* Imagelist (list) = has the form [string_1, string_2, ..., string_n]. Each list element string_i is the complete filename (including path) of one FITS image file.
	* bitpix (integer) = This number defines the data format in the FITS file. If = 16 then the data is saved as 16 bits integer (BITPIX=16 in the FITS header). If = -32 the data is saved as 32 bits float (BITPIX=-32 in the FITS header). Default value is 16. 
	* overwrite (boolean) = optional argument, default value is True. If False, the script asks the user to confirm to save each image, if a file already exists with the same name.
	---------------------
	Output = no output
	---------------------
	"""
	
	#--- conditioning array dimensions ---
	if len(images.shape)==2:
		images = np.tile(images,(1,1,1))	

	#--- conditioning data to unsigned 16bits integer / 32bits float ---
	if bitpix==16:
		test = (images<65535.)
		images = (1.-test)*65535. + test*images		
		images = (images>0)*images				
		images = np.array(np.round(images,0),dtype=np.uint16)
	elif bitpix==-32:
		images = np.array(images,dtype=np.float32)
		
	#--- creating FITS and saving ---		
	for i in range(images.shape[0]):
		#--- create FITS ---
		hdu = astropy.io.fits.PrimaryHDU()
		hdu.data = np.array(images[i,:,:])

		#managing FITS data offset and scale
		if bitpix==16:
			hdu.scale('int16','', bzero=32768)
			hdu.header['BZERO']=(32768.,"")
			hdu.header['BSCALE']=(1.,"")

		#adding date of file writing
		now = datetime.datetime.now()
		date = now.strftime("%Y-%m-%dT%H:%M:%S")
		hdu.header["DATE"]=(str(date),'Date of file writing')

		#writing all other headers
		for hh in sorted(headers[i].keys()):
			if (hh!='BZERO')and(hh!='BSCALE')and(hh!='DATE')and(hh!='SIMPLE')and(hh!='BITPIX')and(hh!='NAXIS')and(hh!='NAXIS1')and(hh!='NAXIS2')and(hh!='EXTEND'):
				hdu.header[hh]=(headers[i][hh][0],headers[i][hh][1])
			
		#--- save FITS ---
		if os.path.exists(Imagelist[i]):
			if overwrite:
				os.remove(Imagelist[i])
			else:
				print("")
				print("aspylib.astro.save_imagelist():")				
				if (sys.version[0]=="2"):
					ans = raw_input("File "+Imagelist[i]+" already exists. Do you want to overwrite it (yes/no)?")
				elif (sys.version[0]=="3"):
					ans = input("File "+Imagelist[i]+" already exists. Do you want to overwrite it (yes/no)?")
				if ans=='yes':
					os.remove(Imagelist[i])
				else:
					sys.exit("Script execution stopped")
		hdu.writeto(Imagelist[i])
	#-------------------------------------

