# -*- coding: iso_8859_1 -*-

import numpy as np
import platform as pf
import aspylib.conf as conf

from ctypes import c_long, c_float, cdll
from numpy.ctypeslib import ndpointer 

def translate_Bspline(images, shift, spline_degree):
	"""
	---------------------
	Purpose
	Translates an image (or list of images) by non-integer shifts in both X and Y directions.
	B-Spline interpolation with arbitrary order (from 2 to 9) is used to calculate the translated pixels.
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = array that contains the data of one image (2D case) or the data of a list of images (3D case).
	* shift ([float, float]) = 2 elements list that contains respectively the shifts in X (vertical) and Y (horizontal) directions.
	* spline_degree (integer) = the degree of the B-Spline that is used for interpolation : it can be 2, 3, 4, 5, 6, 7, 8 or 9.
	---------------------
	Output (2D or 3D Numpy array) = transformed image data.
	---------------------
	"""

	ierror = 0
	while True:
		try:
			if len(images.shape)==2:
				dim = 1
				dim_x, dim_y = images.shape
			else:
				dim, dim_x, dim_y = images.shape
				
			indata = np.ascontiguousarray(np.array(images, dtype = np.float32))
			dxy = np.ascontiguousarray(np.array(shift, dtype = np.float32))
			outdata = np.ascontiguousarray(np.empty(images.shape, dtype = np.float32))
			
			if pf.system()=='Windows':
				cdll.Bspline.Translate.argtypes = [c_long, c_long, c_long, ndpointer(dtype=np.float32), ndpointer(dtype=np.float32), c_long, ndpointer(dtype=np.float32)]
				cdll.Bspline.Translate(dim,dim_x,dim_y,indata,dxy,spline_degree,outdata)
			elif pf.system()=='Linux':
				mydll = cdll.LoadLibrary("libbspline.so")
				mydll.Translate.argtypes = [c_long, c_long, c_long, ndpointer(dtype=np.float32), ndpointer(dtype=np.float32), c_long, ndpointer(dtype=np.float32)]
				mydll.Translate(dim,dim_x,dim_y,indata,dxy,spline_degree,outdata)

			del indata
			del dxy
			break
		except:
			print("error in aspylib.astro.translate_Bspline()")
			ierror+=1
			if ierror>3:
				raise Exception("more than 3 consecutive failures, execution stopped")
			pass

	return outdata

def rotate_Bspline(images, xy_axis, angle, spline_degree):
	"""
	---------------------
	Purpose
	Rotates an image (or list of images) by an arbitrary angle, with an arbitrary (non-integer, possibly outside the image) position of the rotation axis.
	B-Spline interpolation with arbitrary order (from 2 to 9) is used to calculate the rotated pixels.
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = array that contains the data of one image (2D case) or the data of a list of images (3D case).
	* xy_axis ([float, float]) = 2 elements list that contains respectively the X (vertical) and Y (horizontal) positions of the rotation axis.
	* angle (float) = rotation angle in degrees, measured clockwise.
	* spline_degree (integer) = the degree of the B-Spline that is used for interpolation : it can be 2, 3, 4, 5, 6, 7, 8 or 9.
	---------------------
	Output (2D or 3D Numpy array) = transformed image data.
	---------------------
	"""

	ierror = 0
	while True:
		try:
			if len(images.shape)==2:
				dim = 1
				dim_x, dim_y = images.shape
			else:
				dim, dim_x, dim_y = images.shape

			indata = np.ascontiguousarray(np.array(images, dtype = np.float32))
			axis = np.ascontiguousarray(np.array(xy_axis, dtype = np.float32))
			outdata = np.ascontiguousarray(np.empty(images.shape, dtype = np.float32))

			if pf.system()=='Windows':
				cdll.Bspline.Rotate.argtypes = [c_long, c_long, c_long, ndpointer(dtype=np.float32), ndpointer(dtype=np.float32), c_float, c_long, ndpointer(dtype=np.float32)]
				cdll.Bspline.Rotate(dim,dim_x,dim_y,indata,axis,np.float32(angle),spline_degree,outdata)
			elif pf.system()=='Linux':
				mydll = cdll.LoadLibrary("libbspline.so")
				mydll.Rotate.argtypes = [c_long, c_long, c_long, ndpointer(dtype=np.float32), ndpointer(dtype=np.float32), c_float, c_long, ndpointer(dtype=np.float32)]
				mydll.Rotate(dim,dim_x,dim_y,indata,axis,np.float32(angle),spline_degree,outdata)

			del indata
			del axis
			break
		except:
			print("error in aspylib.astro.rotate_Bspline()")
			ierror+=1
			if ierror>3:
				raise Exception("more than 3 consecutive failures, execution stopped")
			pass

	return outdata

def scale_Bspline(images, xy_center, scaling, spline_degree):
	"""
	---------------------
	Purpose
	Scales an image (or list of images) by an arbitrary factor (larger or smaller than 1), with an arbitrary (non-integer, possibly outside the image) position of the stationary pixel.
	B-Spline interpolation with arbitrary order (from 2 to 9) is used to calculate the scaled pixels.
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = array that contains the data of one image (2D case) or the data of a list of images (3D case).
	* xy_center ([float, float]) = 2 elements list that contains respectively the X (vertical) and Y (horizontal) positions of the stationary pixel.
	* scaling (float, must be >0) = scaling factor.
	* spline_degree (integer) = the degree of the B-Spline that is used for interpolation : it can be 2, 3, 4, 5, 6, 7, 8 or 9.
	---------------------
	Output (2D or 3D Numpy array) = transformed image data.
	---------------------
	"""

	ierror = 0
	while True:
		try:
			if len(images.shape)==2:
				dim = 1
				dim_x, dim_y = images.shape
			else:
				dim, dim_x, dim_y = images.shape

			indata = np.ascontiguousarray(np.array(images, dtype = np.float32))
			center = np.ascontiguousarray(np.array(xy_center, dtype = np.float32))
			outdata = np.ascontiguousarray(np.empty(images.shape, dtype = np.float32))

			if pf.system()=='Windows':
				cdll.Bspline.Scale.argtypes = [c_long, c_long, c_long, ndpointer(dtype=np.float32), ndpointer(dtype=np.float32), c_float, c_long, ndpointer(dtype=np.float32)]
				cdll.Bspline.Scale(dim,dim_x,dim_y,indata,center,np.float32(scaling),spline_degree,outdata)
			elif pf.system()=='Linux':
				mydll = cdll.LoadLibrary("libbspline.so")
				mydll.Scale.argtypes = [c_long, c_long, c_long, ndpointer(dtype=np.float32), ndpointer(dtype=np.float32), c_float, c_long, ndpointer(dtype=np.float32)]
				mydll.Scale(dim,dim_x,dim_y,indata,center,np.float32(scaling),spline_degree,outdata)

			del indata
			del center
			break
		except:
			print("error in aspylib.astro.scale_Bspline()")
			ierror+=1
			if ierror>3:
				raise Exception("more than 3 consecutive failures, execution stopped")
			pass
			
	return outdata

def translate_bilinear(images, shift):
	"""
	---------------------
	Purpose
	Translates an image (or list of images) by non-integer shifts in both X and Y directions.
	Bilinear interpolation is used to calculate the translated pixels.
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = array that contains the data of one image (2D case) or the data of a list of images (3D case).
	* shift ([float, float]) = 2 elements list that contains respectively the shifts in X (vertical) and Y (horizontal) directions.
	---------------------
	Output (2D or 3D Numpy array) = transformed image data.
	---------------------
	"""

	ierror = 0
	while True:
		try:
			if len(images.shape)==2:
				dim = 1
				dim_x, dim_y = images.shape
			else:
				dim, dim_x, dim_y = images.shape

			indata = np.ascontiguousarray(np.array(images, dtype = np.float32))
			dxy = np.ascontiguousarray(np.array(shift, dtype = np.float32))
			outdata = np.ascontiguousarray(np.empty(images.shape, dtype = np.float32))
			
			if pf.system()=='Windows':
				cdll.bilinear.Translate.argtypes = [c_long, c_long, c_long, ndpointer(dtype=np.float32), ndpointer(dtype=np.float32), ndpointer(dtype=np.float32)]
				cdll.bilinear.Translate(dim,dim_x,dim_y,indata,dxy,outdata)
			elif pf.system()=='Linux':
				mydll = cdll.LoadLibrary("libbilinear.so")
				mydll.Translate.argtypes = [c_long, c_long, c_long, ndpointer(dtype=np.float32), ndpointer(dtype=np.float32), ndpointer(dtype=np.float32)]
				mydll.Translate(dim,dim_x,dim_y,indata,dxy,outdata)

			del indata
			del dxy
			break
		except:
			print("error in aspylib.astro.translate_bilinear()")
			ierror+=1
			if ierror>3:
				raise Exception("more than 3 consecutive failures, execution stopped")
			pass
			
	return outdata

def rotate_bilinear(images, xy_axis, angle):
	"""
	---------------------
	Purpose
	Rotates an image (or list of images) by an arbitrary angle, with an arbitrary (non-integer, possibly outside the image) position of the rotation axis.
	Bilinear interpolation is used to calculate the rotated pixels.
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = array that contains the data of one image (2D case) or the data of a list of images (3D case).
	* xy_axis ([float, float]) = 2 elements list that contains respectively the X (vertical) and Y (horizontal) positions of the rotation axis.
	* angle (float) = rotation angle in degrees, measured clockwise.
	---------------------
	Output (2D or 3D Numpy array) = transformed image data.
	---------------------
	"""

	ierror = 0
	while True:
		try:
			if len(images.shape)==2:
				dim = 1
				dim_x, dim_y = images.shape
			else:
				dim, dim_x, dim_y = images.shape

			indata = np.ascontiguousarray(np.array(images, dtype = np.float32))
			axis = np.ascontiguousarray(np.array(xy_axis, dtype = np.float32))
			outdata = np.ascontiguousarray(np.empty(images.shape, dtype = np.float32))

			if pf.system()=='Windows':
				cdll.bilinear.Rotate.argtypes = [c_long, c_long, c_long, ndpointer(dtype=np.float32), ndpointer(dtype=np.float32), c_float, ndpointer(dtype=np.float32)]
				cdll.bilinear.Rotate(dim,dim_x,dim_y,indata,axis,np.float32(angle),outdata)
			elif pf.system()=='Linux':
				mydll = cdll.LoadLibrary("libbilinear.so")
				mydll.Rotate.argtypes = [c_long, c_long, c_long, ndpointer(dtype=np.float32), ndpointer(dtype=np.float32), c_float, ndpointer(dtype=np.float32)]
				mydll.Rotate(dim,dim_x,dim_y,indata,axis,np.float32(angle),outdata)

			del indata
			del axis
			break
		except:
			print("error in aspylib.astro.rotate_bilinear()")
			ierror+=1
			if ierror>3:
				raise Exception("more than 3 consecutive failures, execution stopped")
			pass

	return outdata

def scale_bilinear(images, xy_center, scaling):
	"""
	---------------------
	Purpose
	Scales an image (or list of images) by an arbitrary factor (larger or smaller than 1), with an arbitrary (non-integer, possibly outside the image) position of the stationary pixel.
	Bilinear interpolation is used to calculate the scaled pixels.
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = array that contains the data of one image (2D case) or the data of a list of images (3D case).
	* xy_center ([float, float]) = 2 elements list that contains respectively the X (vertical) and Y (horizontal) positions of the stationary pixel.
	* scaling (float, must be >0) = scaling factor.
	---------------------
	Output (2D or 3D Numpy array) = transformed image data.
	---------------------
	"""

	ierror = 0
	while True:
		try:
			if len(images.shape)==2:
				dim = 1
				dim_x, dim_y = images.shape
			else:
				dim, dim_x, dim_y = images.shape

			indata = np.ascontiguousarray(np.array(images, dtype = np.float32))
			center = np.ascontiguousarray(np.array(xy_center, dtype = np.float32))
			outdata = np.ascontiguousarray(np.empty(images.shape, dtype = np.float32))

			if pf.system()=='Windows':
				cdll.bilinear.Scale.argtypes = [c_long, c_long, c_long, ndpointer(dtype=np.float32), ndpointer(dtype=np.float32), c_float, ndpointer(dtype=np.float32)]
				cdll.bilinear.Scale(dim,dim_x,dim_y,indata,center,np.float32(scaling),outdata)
			elif pf.system()=='Linux':
				mydll = cdll.LoadLibrary("libbilinear.so")
				mydll.Scale.argtypes = [c_long, c_long, c_long, ndpointer(dtype=np.float32), ndpointer(dtype=np.float32), c_float, ndpointer(dtype=np.float32)]
				mydll.Scale(dim,dim_x,dim_y,indata,center,np.float32(scaling),outdata)

			del indata
			del center
			break
		except:
			print("error in aspylib.astro.scale_bilinear()")
			ierror+=1
			if ierror>3:
				raise Exception("more than 3 consecutive failures, execution stopped")
			pass

	return outdata

def bin2x2(images, saturation=True):
	"""
	---------------------
	Purpose
	This function performs a 2x2 binning of a list of FITS images.
	A binned pixel is calculated with a factor 0.5, for instance binned_pixel(1,1) = 0.25*(pixel(1,1)+pixel(1,2)+pixel(2,1)+pixel(2,2)).
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = array that contains the data of one image (2D case) or the data of a list of images (3D case).
	* saturation (boolean) = default value is True. If True, the pixels of the binned images are set to the saturation level whenever at least one of the 4 corresponding pixels in the unbinned image are saturated.
	---------------------
	Output (2D or 3D Numpy array) = binned image data.
	---------------------
	"""
	
	ierror = 0
	while True:
		try:
			if len(images.shape)==2:
				dim = 1
				dim_x, dim_y = images.shape
				outshape = (int(dim_x/2), int(dim_y/2))
			else:
				dim, dim_x, dim_y = images.shape
				outshape = (dim, int(dim_x/2), int(dim_y/2))

			indata = np.ascontiguousarray(np.array(images, dtype = np.float32))
			outdata = np.ascontiguousarray(np.empty(outshape, dtype = np.float32))
			if saturation:
				sat = int(conf.saturation)
			else:
				sat = 0

			if pf.system()=='Windows':
				cdll.binning.Binning2x2.argtypes = [c_long, c_long, c_long, ndpointer(dtype=np.float32), ndpointer(dtype=np.float32), c_long]
				cdll.binning.Binning2x2(dim,dim_x,dim_y,indata,outdata,sat)
			elif pf.system()=='Linux':
				mydll = cdll.LoadLibrary("libbinning.so")
				mydll.Binning2x2.argtypes = [c_long, c_long, c_long, ndpointer(dtype=np.float32), ndpointer(dtype=np.float32), c_long]
				mydll.Binning2x2(dim,dim_x,dim_y,indata,outdata,sat)

			del indata
			break
		except:
			print("error in aspylib.astro.bin2x2()")
			ierror+=1
			if ierror>3:
				raise Exception("more than 3 consecutive failures, execution stopped")
			pass

	return outdata
