# -*- coding: iso_8859_1 -*-

import numpy as np
import platform as pf
from ctypes import byref, c_double, cdll, POINTER, c_int

def matching_pairs(stars1, stars2, distcrit):
	"""
	---------------------
	Purpose
	Match two lists of star positions, given a distance threshold.
	The function returns two arrays of indices x=[x1,x2,..,xn] and y=[y1,y2,..,yn] such that the positions stars1[xi] and stars2[yi] are closer than the distance criterium and have been identified as corresponding to the same star. A specific position may not appear more than once in the returned lists.
	Special cases have been treated with care, for instance: two positions in stars1 within short distance of a single position in stars2. In such case the closest pair is kept. Be careful that astrometry.matching_pairs(stars1,stars2,distcrit) and astrometry.matching_pairs(stars2,stars1,distcrit) will return the same result in most cases but not in all cases.
	---------------------
	Inputs
	* stars1 (2D Numpy array) = first list of star positions. stars1[:,0] corresponds to the X (vertical) coordinates and stars1[:,1] to the Y (horizontal) coordinates.
	* stars2 (2D Numpy array) = second list of star positions, with the same structure as stars1.
	* distcrit (integer) = distance threshold used for matching the two lists.
	---------------------
	Outputs = x, y
	* x (Numpy array) = array of indices [x1,x2,...,xn] as described above.
	* y (Numpy array) = array of indices [y1,y2,...,yn] as described above.
	---------------------
	"""

	c_double_p = POINTER(c_double)

	x1 = stars1[:,0].astype(np.double)
	x1p = x1.ctypes.data_as(POINTER(c_double_p))

	y1 = stars1[:,1].astype(np.double)
	y1p = y1.ctypes.data_as(POINTER(c_double_p))

	x2 = stars2[:,0].astype(np.double)
	x2p = x2.ctypes.data_as(POINTER(c_double_p))

	y2 = stars2[:,1].astype(np.double)
	y2p = y2.ctypes.data_as(POINTER(c_double_p))

	pairs = c_int * max(len(stars1),len(stars2)) 
	z1=pairs()
	z2=pairs()
	
	dim = c_int()
	
	if pf.system()=='Windows':
		cdll.astrometry.matching_pairs(c_double(distcrit), len(stars1),x1p, y1p, len(stars2), x2p, y2p, byref(dim), byref(z1), byref(z2))
	elif pf.system()=='Linux':
		mydll = cdll.LoadLibrary("libastro.so")
		mydll.matching_pairs(c_double(distcrit), len(stars1),x1p, y1p, len(stars2), x2p, y2p, byref(dim), byref(z1), byref(z2))
	
	x = []
	x[:] = z1[:dim.value]
	y = []
	y[:] = z2[:dim.value]

	return np.array(x),np.array(y)
