# -*- coding: iso_8859_1 -*-

import numpy as np
import matplotlib.pyplot as plt

from scipy import optimize
from aspylib import astro
from aspylib.astrometry import astrometry2, astrometry3, astrometry5

def find_astrometry_for_image(image, proj, distcrit = 5, imzones1 = [1,1], imzones2 = [1,1], nbdetstars = [60,100], nbcatstars = [200,1000], text_display = True, plotting=True, quality_indicators=False):
	"""
	---------------------
	Purpose
	Find a 3rd order polynomial transformation law (20 coefficients) that allows to accurately match an image with a star catalog.
	The calculated law transforms pixel coordinates of one star on the image, into the orthographic projection of the equatorial coordinates of the corresponding star in the catalog. See the functions astrometry.project_stars() and astrometry.inverse_project_stars() that link the orthographic projected coordinates with the equatorial coordinates RA and Dec.
	Three iterations are needed to get accurate results, since no a priori information on the transformation is provided. 
	First an approximate law is computed (4 coefficients such that projX=scaling*(X*cos(phi)-Y*sin(phi))+dprojX, projY=scaling*(X*sin(phi)+Y*cos(phi))+dprojY) based on a cross-correlation method. Then pairs are identified between the image and the catalog positions that are equally distributed over the full image, and the approximate law is refined. Finally, pairs are re-identified over the full image and an accurate law is computed (20 coefficients such that projX = a0+a1*X+a2*Y+a3*X^2+a4*X*Y+a5*Y^2+a6*X^3+a7*X^2*Y+a8*X*Y^2+a9*Y^3, and a similar law for projY).
	The algorithm works only if the North direction is in the two upper quadrants of the image (top of the image +/- 90 deg), otherwise it fails.
	---------------------
	Inputs
	* image (2D Numpy array) = image data.
	* proj (2D Numpy array) = orthographic projected coordinates of the catalog stars, such that proj[:,0] = vertical projected coordinates, proj[:,1] = horizontal projected coordinates.
	* distcrit (integer) = optional argument, default value is 5. This parameter represents a distance, expressed in image pixel coordinates, used as a threshold to identify pairs between the image positions and the catalog positions. If one image position is closer than distcrit from one catalog position, then both are assumed to correspond to the same star for the astrometric reduction.
	* imzones1 (list with two integers) = optional argument, default value is [1,1]. If different than [1,1], the image is seperated in imzones1[0]*imzones1[1] areas of equal size, with imzones1[0] areas in the vertical direction and imzones1[1] areas in the horizontal direction. For computing the first estimate of the approximate law, nbdetstars[0] stars are detected over each of these zones, for a total of nbdetstars[0]*imzones1[0]*imzones1[1] detected stars. This parameter allows to use stars equally distributed over the image to get better results with the astrometric reduction.
	* imzones2 (list with two integers) = optical argument, default value is [1,1]. Same as imzones1 for detecting stars that will be used for the refinement of the approximate law and for computing the accurate law (nbdetstars[1]*imzones2[0]*imzones2[1] in total).
	* nbdetstars (list with two integers) = optional argument, default value is [60,100]. nbdetstars[0] (resp. nbdetstars[1]) is the number of stars detected on the image for computing the first estimate of the approximate law (resp. for refining the approximate law and computing the accurate law).
	* nbcatstars (list with two integers) = optional argument, default value is [200,1000]. nbcatstars[0] (resp. nbcatstars[1]) is the number of stars taken from the catalog for computing the first estimate of the approximate law (resp. for refining the approximate law and computing the accurate law).
	Note: if the function fails to calculate the matching, in some cases this may be fixed by setting nbdetstars[0], nbdetstars[1], nbcatstars[0] or nbcatstars[1] to different values.
	* text_display (boolean) = optional argument. If equal to True (default), some information is written in the console window. If equal to False nothing is written.
	* plotting (boolean) = optional argument. If equal to True (default), the script is paused to display a plot showing the matching between stars from the image and the catalog. If equal to False no plot is shown.
	* quality_indicators (boolean) = optional argument. Default value is False. If equal to True the script returns a list of values to assess the quality of the astrometric reduction (see description below).
	---------------------
	Outputs = newcoef, indicators
	* newcoef (list) = List of transformation coefficients [[scaling, phi, dx, dy], [a0,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13,a14,a15,a16,a17,a18,a19]] for both the approximate and accurate laws.
	Note: the rotation angle phi is expressed in radians.
	* indicators (list) = only returned if quality_indicators=True. List of values indicating the quality of the astrometric reduction. Contains: number of matched stars for computing the first estimate of the approximate law + number of matched stars for refining the approximate law + number of matched stars for computing the accurate law + median values of the residual distances for the accurate law, expressed in image pixels and calculated over the stars used for the reduction within 12 parts of the image, between the catalog positions projected onto the image and the image positions.
	---------------------
	"""
	if text_display:
		print("")
		print("aspylib.astrometry.find_astrometry_for_image():")
	
	indicators = []		#indicators for quality of astrometric reduction

	#*********************************************************************************************************
	#		(1) preparing inputs
	#*********************************************************************************************************

	#--- catalog subsets ---
	proj1 = proj[:nbcatstars[0],:]
	proj2 = proj[:nbcatstars[1],:]

	#--- finding stars on image ---
	imsize = np.shape(image)

	xind = np.linspace(0,imsize[0]-1,imzones1[0]+1)
	yind = np.linspace(0,imsize[1]-1,imzones1[1]+1)
	xind = [[int(np.ceil(xind[i])),int(np.floor(xind[i+1]))] for i in range(imzones1[0])]
	yind = [[int(np.ceil(yind[i])),int(np.floor(yind[i+1]))] for i in range(imzones1[1])]
	stars1 = []
	for xx in xind:
		for yy in yind:
			xy = astro.find_stars(image[xx[0]:xx[1],yy[0]:yy[1]], nbdetstars[0], text_display=False)+np.tile([xx[0],yy[0]],[nbdetstars[0],1])
			stars1.append(xy)
	stars1 = np.vstack(stars1)
	stars1 = astro.centroid_stars(image, stars1, 2)
	stars1, indices = astro.remove_duplicates(stars1)

	xind = np.linspace(0,imsize[0]-1,imzones2[0]+1)
	yind = np.linspace(0,imsize[1]-1,imzones2[1]+1)
	xind = [[int(np.ceil(xind[i])),int(np.floor(xind[i+1]))] for i in range(imzones2[0])]
	yind = [[int(np.ceil(yind[i])),int(np.floor(yind[i+1]))] for i in range(imzones2[1])]
	stars2 = []
	for xx in xind:
		for yy in yind:
			xy = astro.find_stars(image[xx[0]:xx[1],yy[0]:yy[1]], nbdetstars[1], text_display=False)+np.tile([xx[0],yy[0]],[nbdetstars[1],1])
			stars2.append(xy)
	stars2 = np.vstack(stars2)
	stars2 = astro.centroid_stars(image, stars2, 2)
	stars2, indices = astro.remove_duplicates(stars2)

	#*********************************************************************************************************
	#		(2) first astrometric solution
	#*********************************************************************************************************

	#--- approximate solution ---
	#scaling + rotation + translation that transform the pixel coordinates into stereographic coordinates
	scaling, phi, dx, dy = astrometry2.approximate_astrometry(stars1, proj1, nb_iterations=2, resolution=1024)
	newcoef = [[scaling, phi, dx, dy],[]]		
	
	#*********************************************************************************************************
	#		(3) checking + display results
	#*********************************************************************************************************
	
	#--- star transformation ---
	trans1 = astrometry5.transform_stars(newcoef[0], stars1)
	#--- calculate matching pairs ---
	x,y = astrometry3.matching_pairs(trans1,proj1,distcrit*newcoef[0][0])
	indicators.append(len(x))

	#--- checking ---
	if text_display:
		print("")
		print("--- searching approximate astrometry ---")
		print("nb of stars used (detected / from catalog):",len(stars1),"/",len(proj1))
		print("approximate astrometric coefficients:", scaling, phi, dx, dy)
		print("nb of matched pairs:" ,len(x))
	
	if plotting:
		plt.figure()
		plt.scatter(trans1[:,1],trans1[:,0],s=20)
		plt.scatter(proj1[:,1],proj1[:,0],s=20,marker='+',color='r')
		plt.grid()
		plt.title("first star matching")
		plt.show()

	if len(x)<10:
		print("")
		print("aspylib.astrometry.find_astrometry_for_image():")
		print("-----------------------------------------------------------------------")
		print("astrometric reduction failed, with only "+str(len(x))+" matching stars")
		print("-----------------------------------------------------------------------")
		print("")
		raise Exception("Failed astrometry in aspylib.astrometry.find_astrometry_for_image()")

	#*********************************************************************************************************
	#		(4) accurate astrometry
	#*********************************************************************************************************
	
	#--- star transformation ---
	trans2 = astrometry5.transform_stars(newcoef[0], stars2)
	#--- calculate matching pairs ---
	x,y = astrometry3.matching_pairs(trans2,proj2,distcrit*newcoef[0][0])
	indicators.append(len(x))

	#--- refining approximate astrometry ---
	p0 = newcoef[0]
	coordinates = np.vstack([stars2[x,0], stars2[x,1], proj2[y,0], proj2[y,1]]).transpose()
	errfunc = lambda p, xy: (astrometry5.transform_stars(p, xy[:,:2]) - xy[:,2:]).flatten() # Distance to the target function
	p1, success = optimize.leastsq(errfunc, p0, args=(coordinates,))
	scaling, phi, dx, dy = p1
	newcoef = [[scaling, phi, dx, dy],[]]		

	#--- star transformation ---
	trans2 = astrometry5.transform_stars(newcoef[0], stars2)
	#--- calculate matching pairs ---
	x,y = astrometry3.matching_pairs(trans2,proj2,distcrit*newcoef[0][0])
	indicators.append(len(x))
	
	if text_display:
		print("")
		print("--- refining approximate astrometry ---")
		print("nb of stars used (detected / from catalog):",len(stars2),"/",len(proj2))
		print("nb of matched pairs:" ,indicators[-2])
		print("new astrometric coefficients:", scaling, phi, dx, dy)
		print("new nb of matched pairs:" ,indicators[-1])

	#--- accurate astrometry ---
	if len(x)<10:
		print("")
		print("aspylib.astrometry.find_astrometry_for_image():")
		print("-----------------------------------------------------------------------")
		print("searching accurate astrometry:")
		print("not enough matching pairs for least square fit, an error will occur")
		print("-----------------------------------------------------------------------")
		
	p0 = [dx, scaling*np.cos(phi), -scaling*np.sin(phi), 0., 0., 0., 0., 0., 0., 0., dy, scaling*np.sin(phi), scaling*np.cos(phi), 0., 0., 0., 0., 0., 0., 0.]
	scalingfactor = np.array([1e5,1.,1.,1e-6,1e-6,1e-6,1e-9,1e-9,1e-9,1e-9]*2)
	p0 /= scalingfactor
	coordinates = np.vstack([stars2[x,0], stars2[x,1], proj2[y,0], proj2[y,1]]).transpose()
	errfunc = lambda p, xy, scalingfactor: (astrometry5.distorsion_law(p, xy[:,:2], scaling=scalingfactor) - xy[:,2:]).flatten() # Distance to the target function
	p1, success = optimize.leastsq(errfunc, p0, args=(coordinates,scalingfactor,))
	p1 *= scalingfactor
	newcoef[1] = p1

	#--- added 04-05-2013 ---
	error = (astrometry5.inverse_distorsion_law(p1, coordinates[:,2:]) - coordinates[:,:2])
	error = np.sqrt(error[:,0]**2.+error[:,1]**2.)
	
	imzones = [3,4]
	xind = np.linspace(0,imsize[0]-1,imzones[0]+1)
	yind = np.linspace(0,imsize[1]-1,imzones[1]+1)
	xind = [[int(np.ceil(xind[i])),int(np.floor(xind[i+1]))] for i in range(imzones[0])]
	yind = [[int(np.ceil(yind[i])),int(np.floor(yind[i+1]))] for i in range(imzones[1])]
	for xx in xind:
		for yy in yind:
			test = (coordinates[:,0]>xx[0])*(coordinates[:,0]<xx[1])*(coordinates[:,1]>yy[0])*(coordinates[:,1]<yy[1])
			if np.sum(test)>0:
				indicators += [np.ma.median(error[test])]
			else:
				indicators += [np.nan]

	#*********************************************************************************************************
	#		(5) display results
	#*********************************************************************************************************

	if text_display:
		print("")
		print("--- searching accurate astrometry ---")
		print("nb of stars used (detected / from catalog):",len(stars2),"/",len(proj2))
		print("nb of matched pairs used for accurate astrometry:" ,len(x))
		print("astrometric coefficients:", p1)

	# if plotting:
		# plt.figure()
		# plt.scatter(trans2[:,1],trans2[:,0],s=20)
		# plt.scatter(proj2[:,1],proj2[:,0],s=20,marker='+',color='r')
		# plt.grid()
		# plt.title("accurate star matching")
		# plt.show()

	if quality_indicators:
		return newcoef, indicators
	else:
		return newcoef
