# -*- coding: iso_8859_1 -*-

import sys
if (sys.version[0]=="2"):
	import urllib as urlmodule
elif (sys.version[0]=="3"):
	import urllib.request as urlmodule
from aspylib import conf
from aspylib.astrometry import query2

def send_query_vizier(catalog, position, conesize, constraint, filename = conf.folder+'vizier.xml', text_display=True):
	"""
	---------------------
	Purpose
	Send a query to Vizier and retrieves a list of stars with astrometric (alpha, delta, etc) or photometric (magnitudes, etc) information
	The data is saved in a specific xml file, with VOtable format
	The syntax to send a Vizier query is described there: http://vizier.u-strasbg.fr/doc/asu-summary.htx
	---------------------
	Inputs
	* catalog (string) = name of the catalog, following Vizier syntax
	Popular Vizier catalogs include:
	'II/294' - Sloan SDSS photometric catalog Release 7 (2009)
	'2MASS-PSC' - 2MASS point source catalog (2003)
	'GSC2.3' - Version 2.3.2 of the HST Guide Star Catalog (2006)
	'USNO-B1' - Verson B1 of the US Naval Observatory catalog (2003)
	'NVSS'  - NRAO VLA Sky Survey (1998)
	'B/DENIS/DENIS' - 2nd Deep Near Infrared Survey of southern Sky
	'I/259/TYC2' - Tycho-2 main catalog (2000)
	'I/311/HIP2' - Hipparcos main catalog, new reduction (2007)	Each string gives the complete filename (including path) of a file that contains the lightcurve measurements.
	* position (string or [float,float]) = position name (resolved by Sesame) or alpha/delta position (in degrees) giving the center of the star field to be requested
	* conesize (None, float or [float,float]) = size in arcmin, corresponds to the radius (float) or alpha/delta box rectangular size ([float,float]) defining the star field requested
	If equal to None (default), a radius of 5 arcmin is used
	* constraint (string) = list of constraints to be applied on the catalogs fields to filter the retrieved stars (Vizier syntax)
	For instance: 'B1mag<=13'
	Multiple contraints are separated by comma: 'B1mag<13,B2mag<14'
	* filename (string) = optional parameter, default is 'vizier.xml'.
	Specifies the xml file in which the data from the Vizier request is stored.
	It can be a full filename with path information, or a simple filename in which case the active directory is used.
	* text_display (boolean) = optional argument. If equal to True (default), some information is written in the console window. If equal to False nothing is written.
	---------------------
	Output = no output
	---------------------
	"""
	#------ (1) Preparing url for Vizier ----------------------

	cat = "?-source=" + catalog
	if isinstance(position,str):
		obj = "&-c=" + position
	else:
		obj = "&-c.ra=" + str(position[0]) + "&-c.dec=" + str(position[1])
	if conesize==None:
		dis = "&-c.rm=05"
	elif isinstance(conesize,list):
		dis = "&-c.bm=" + str(conesize[0]) + "/" + str(conesize[1])
	else:
		dis = "&-c.rm=" + str(conesize)
	if len(constraint)==0:
		cons = ""
	else:
		constraint = constraint.replace(" ","")
		constraint = constraint.replace(",","&")
		constraint = constraint.replace("<","=%3C")
		constraint = constraint.replace(">","=%3E")
		constraint = constraint.replace("+","%2B")
		constraint = constraint.replace("/","%2F")
		constraint = constraint.replace("!","=!")
		cons = "&" + constraint

	url = "http://webviz.u-strasbg.fr/viz-bin/votable/" + cat + obj + dis + cons + "&-out.max=unlimited"
	#url = "http://vizier.cfa.harvard.edu/viz-bin/votable/" + cat + obj + dis + cons + "&-out.max=unlimited"

	#http://vizier.u-strasbg.fr/viz-bin/votable/VizieR?-source=B/vsx/vsx/&-c.ra=12:12:12&-c.dec=12:12:12&-c.rm=120&-sort=_r&-out.add=_r
	#------ (2) Sending query to Vizier ----------------------
	if text_display:
		print("")
		print("aspylib.astrometry.send_query_vizier():")
		print("--- retrieving data from Vizier ---")

	urlmodule.urlretrieve(url, filename)
	#----------- end of function -----------------------------

def send_query_skybot(date, position, conesize, obscode='500', outputlevel=1, filename = conf.folder+'skybot.xml', text_display=True):
	"""
	---------------------
	Purpose
	Send a query to SkyBoT and retrieves a list of solar system objects with astrometric (alpha, delta, etc) and photometric (magnitudes, etc) information
	The data is saved in a specific xml file, with VOtable format
	The syntax to send a SkyBoT query is described there: http://vo.imcce.fr/webservices/skybot/?conesearch
	---------------------
	Inputs
	* date (float) = Julian date at which information is requested
	* position ([float,float]) = alpha/delta position (in degrees) giving the center of the sky area for which the request is formulated
	* conesize (None, float or [float,float]) = size in arcmin, corresponds to the radius (float) or alpha/delta box rectangular size ([float,float]) defining the sky area for which the request is formulated
	If equal to None (default), a radius of 5 arcmin is used
	* obscode (string) = optional parameter, default is '500'. Corresponds to the IAU observatory code for which information is requested. '500' corresponds to Earth center (geocentric positions).
	* outputlevel (integer) = optional parameter, default is 1.
	Defines the level of detail requested (see the SkyBoT webpage for more information).
	- 1 corresponds to 'object'
	- 2 corresponds to 'basic'
	- 3 corresponds to 'obs'
	- 4 corresponds to 'all'
	* filename (string) = optional parameter, default is 'skybot.xml'.
	Specifies the xml file in which the data from the SkyBoT request is stored.
	It can be a full filename with path information, or a simple filename in which case the active directory is used.
	* text_display (boolean) = optional argument. If equal to True (default), some information is written in the console window. If equal to False nothing is written.
	---------------------
	Output = no output
	---------------------
	"""

	#------ (1) Preparing url for SkyBoT ----------------------

	dat = "-ep=" + str(date)
	pos = "&-ra=" + str(position[0]) + "&-dec=" + str(position[1])
	if conesize==None:
		cone = "&-rm=05"
	elif isinstance(conesize,list):
		cone = "&-bm=" + str(int(conesize[0])) + "x" + str(int(conesize[1]))
	else:
		cone = "&-rm=" + str(int(conesize))
	if outputlevel ==1:
		out = "&-output=object"
	elif outputlevel ==2:
		out = "&-output=basic"
	elif outputlevel ==3:
		out = "&-output=obs"
	else:
		out = "&-output=all"
	loc = "&-loc=" + obscode

	url = "http://vo.imcce.fr/webservices/skybot/skybotconesearch_query.php?" + dat + pos + cone + "&-mime=votable" + out + loc + "&-from=AsPyLib"
	
	#------ (2) Sending query to SkyBoT ----------------------
	if text_display:
		print("")
		print("aspylib.astrometry.send_query_skybot():")
		print("--- retrieving data from SkyBoT ---")
			
	urlmodule.urlretrieve(url, filename)
	#----------- end of function -----------------------------

def check_vsx(position, conesize, filename = conf.folder+'aavso-vsx.xml', text_display=True):
	"""
	---------------------
	Purpose
	Send a query to Vizier and retrieves the list of the variable stars in AAVSO-VSX within the specified area
	The data is saved in a specific xml file, with VOtable format
	The syntax to send a Vizier query is described there: http://vizier.u-strasbg.fr/doc/asu-summary.htx
	The AAVSO-VSX interface is found there: http://vizier.u-strasbg.fr/viz-bin/VizieR?-source=B/vsx
	---------------------
	Inputs
	* position (string or [float,float]) = position name (resolved by Sesame) or alpha/delta position (in degrees) giving the center of the star field to be requested
	* conesize (None, float or [float,float]) = size in arcmin, corresponds to the radius (float) or alpha/delta box rectangular size ([float,float]) defining the star field requested
	If equal to None (default), a radius of 5 arcmin is used
	* filename (string) = optional parameter, default is 'aavso-vsx.xml'.
	Specifies the xml file in which the data from the Vizier request is stored.
	It can be a full filename with path information, or a simple filename in which case the active directory is used.
	* text_display (boolean) = optional argument. If equal to True (default), some information is written in the console window. If equal to False nothing is written.
	---------------------
	Output = no output
	---------------------
	"""
	#------ (1) Preparing url for Vizier ----------------------

	if isinstance(position,str):
		obj = "&-c=" + position
	else:
		obj = "&-c.ra=" + str(position[0]) + "&-c.dec=" + str(position[1])
	if conesize==None:
		dis = "&-c.rm=05"
	elif isinstance(conesize,list):
		dis = "&-c.bm=" + str(int(conesize[0])) + "/" + str(int(conesize[1]))
	else:
		dis = "&-c.rm=" + str(int(conesize))
		
	url = "http://vizier.u-strasbg.fr/viz-bin/votable/VizieR?-source=B/vsx/vsx/" + obj + dis + "&-sort=_r&-out.add=_r"
	#url = "http://vizier.cfa.harvard.edu/viz-bin/votable/VizieR?-source=B/vsx/vsx/" + obj + dis + "&-sort=_r&-out.add=_r"
	
	#------ (2) Sending query to Vizier ----------------------
	if text_display:
		print("")
		print("aspylib.astrometry.check_vsx():")
		print("--- retrieving data from Vizier ---")

	urlmodule.urlretrieve(url, filename)
	#----------- end of function -----------------------------

def find_const(position, filename = conf.folder+'const.xml', text_display=False):
	"""
	---------------------
	Purpose
	Send a query to Vizier and retrieves the constellation corresponding to a constellation
	The data is saved in a specific xml file "const.xml", with VOtable format
	The syntax to send a Vizier query is described there: http://vizier.u-strasbg.fr/doc/asu-summary.htx
	The interface to find constellations is found there: http://vizier.u-strasbg.fr/vizier/VizieR/constellations.htx
	---------------------
	Inputs
	* position (string or [float,float]) = position name (resolved by Sesame) or alpha/delta position (in degrees) giving the center of the star field to be requested (equinox J2000.0 is assumed)
	* filename (string) = optional parameter, default is 'const.xml'.
	Specifies the xml file in which the data from the Vizier request is stored.
	It can be a full filename with path information, or a simple filename in which case the active directory is used.	
	* text_display (boolean) = optional argument. If equal to True (default), some information is written in the console window. If equal to False nothing is written.
	---------------------
	Output (string) = Constellation designation
	---------------------
	"""
	#------ (1) Preparing url for Vizier ----------------------

	if isinstance(position,str):
		obj = "&-c=" + position
	else:
		obj = "&-c.ra=" + str(position[0]) + "&-c.dec=" + str(position[1])
		
	url = "http://vizier.u-strasbg.fr/viz-bin/votable/VizieR?-source=VI/42/out/" + obj + "&-c.eq=J2000"
	#url = "http://vizier.cfa.harvard.edu/viz-bin/votable/VizieR?-source=VI/42/out/" + obj + "&-c.eq=J2000"

	#------ (2) Sending query to Vizier ----------------------
	if text_display:
		print("")
		print("aspylib.astrometry.find_const():")
		print("--- retrieving data from Vizier ---")

	urlmodule.urlretrieve(url,filename)

	#------ (3) Retrieving constellation name ----------------------
	return query2.read_votable(filename, text_display=text_display)[0][2]
	#----------- end of function -----------------------------

def get_image(position, survey, fieldsize, imsize, filename, format, invert = False, grid =False, showtarget= False, text_display = True):
	"""
	---------------------
	Purpose
	Send a query to SkyView and retrieves an image of the requested field
	The data is saved in the specified image name (parameter filename) 
	Information about SkyView can be found at: https://skyview.gsfc.nasa.gov
	---------------------
	Inputs
	* position ([float,float] or [string,string]) = alpha/delta position (in decimal degrees or with format [hh:mm:ss.s,dd:mm:ss.s]) giving the center of the requested star field
	* survey (string) = name of the requested survey. Can be "DSS", "DSS2IR", "DSS2R", etc. See SkyView online help for more information.
	* fieldsize ([float,float]) = size of the alpha/delta rectangular box ([float,float]) defining the star field requested. The requested width does not correspond to alpha but to alpha/cos(delta).
	* imsize ([int,int]) = size of the requested image in pixels.
	* filename (string) = image file name used to save the retrieved image. It can be a full filename with path information, or a simple filename in which case the active directory is used.
	* format (string) = specifies the image format requested. Can be 'fits', 'png', 'jpg'.
	* invert (boolean) = optional argument, only used for graphic image formats (PNG, JPG, but not FITS). If equal to False (default), then stars are white over a dark background. If equal to True then the stars are dark over a white background.
	* grid (boolean) = optional argument, default value is False. When equal to True, a grid of equatorial coordinates is drawn on the image.
	* showtarget (boolean) = optional argument, default value is False. When equal to True, a red circle is drawn at the center of the image to show the target position.
	* text_display (boolean) = optional argument. If equal to True (default), some information is written in the console window. If equal to False nothing is written.
	---------------------
	Output = no output
	---------------------
	"""
	#------ (1) Preparing url ----------------------

	obj = "position=" + str(position[0]) + "," + str(position[1])
	sur = "&survey=" + str(survey)
	siz = "&size=" + str(fieldsize[0]) + "," + str(fieldsize[1])
	ims = "&pixels=" + str(imsize[0]) + "," + str(imsize[1])
	
	if format == 'fits':
		fmt = '&return=fits'
	else:
		fmt = '&return=' + format + '&nofits='

	if invert:
		inv = '&Invert='
	else:
		inv = ''
		
	if grid:
		grd = "&Grid=&Gridlabels="
	else:
		grd = ""
		
	if showtarget:
		offset = str(min(imsize[0], imsize[1]))
		#tgt = "&Draw='color+black," + offset + "0," + offset + "0,color+red,circle+0+0+10'"
		tgt = "&draw=\'color+red,circle+0+0+10\'"
	else:
		tgt = ""
		
	url = "http://skyview.gsfc.nasa.gov/cgi-bin/images?" + obj + sur + siz + ims + "&min=1000&sampler=Clip&projection=Tan&coordinates=J2000.0" + fmt + tgt + inv + grd 

	#------ (2) Sending query ----------------------
	if text_display:
		print("")
		print("aspylib.astrometry.get_image():")
		print("--- retrieving data from SkyView ---")
	
	urlmodule.urlretrieve(url, filename)
	#----------- end of function -----------------------------
