# -*- coding: iso_8859_1 -*-

import numpy as np
from aspylib import astro
from aspylib.calib import calib3

def process_flatframe_constantexp(Flatlist, Darklist, Finalflat, Config, Headers={}):
	"""
	---------------------
	Purpose
	Computes a master flat image from a series of several flat field and dark FITS images, and saves it to a FITS file.
	This script assumes that the same integration time was used for the flat field frames and dark frames.
	---------------------
	Inputs
	* Flatlist (list) = Flat images to be processed. Has the form [string_1, string_2, ..., string_n]. Each list element string_i is the complete filename (including path) of one FITS image.
	* Darklist (list) = Dark images to be processed. Has the form [string_1, string_2, ..., string_n]. Each list element string_i is the complete filename (including path) of one FITS image.
	* Finalflat (list) = one element list containing a string [string]. It gives the complete filename (including path) of the FITS file where the master flat will be saved.
	* Config (integer) = the FITS headers of the final master flat image are copy-pasted from the first image in Flatlist. Config defines which set of rules must be used, additionally, to modify the FITS headers obtained from the image Flatlist[0]. The file conf.py allows to define several sets of rules and specifies which headers must be removed, kept, or added.
	* Headers (dictionary) = optional argument. Default value is an empty dictionary. Gives additional header entries, to be added to the headers of the final master flat image. If one keyword already exists in the headers of Flatlist[0], it will be overwritten by the new value/comment provided with Headers.
	FITS headers are created in three steps in this order:
		(1) headers are read-out from the image Flatlist[0] and modified according to the parameter Config,
		(2) additional header entries are added according to Headers,
		(3) as a last step, some modifications are done for the keywords EXPTIME, EXPOSURE, DATE-OBS and DATE-END: see Python code.
	---------------------
	Output (2D Numpy Array) = data of the final flat image.
	---------------------
	"""
	print("")
	print("aspylib.calib.process_flatframe_constantexp():")

	#------ generating dark ------
	dark_final = astro.add_median(astro.get_imagedata(Darklist))
	hotpixels = calib3.find_hotpixels(dark_final, 100)
	print("dark final generated ... ok")

	#------ generating flat ------
	flats = astro.get_imagedata(Flatlist)
	for i in range(len(Flatlist)):
		flats[i,:,:] = flats[i,:,:] - dark_final	
		flats[i,:,:] = flats[i,:,:] * 20000. / astro.get_median(flats[i,:,:])
	flat_final = astro.add_median(flats)
	del flats
	flat_avg = (np.roll(flat_final,1,axis=0) + np.roll(flat_final,-1,axis=0) + np.roll(flat_final,1,axis=1) + np.roll(flat_final,-1,axis=1)) / 4.
	flat_final = flat_avg*hotpixels + flat_final*(1.-hotpixels)	
	print("flat final generated ... ok")
	print("max value (ADU)=", np.max(flat_final.flatten()))

	#astro.display(flat_final, title='Final flat frame')

	#------ saving final image.fit ------
	headerdict = astro.create_headers(Config, [Flatlist[0]])[0]
	headerdict.update(Headers)
	if astro.get_headervalues([Flatlist[0]], 'EXPOSURE')[0]!='':
		Exp = np.sum(np.array(astro.get_headervalues(Flatlist, 'EXPOSURE')))
	else:
		Exp = np.sum(np.array(astro.get_headervalues(Flatlist, 'EXPTIME')))
	Dateobs = sorted(astro.get_headervalues(Flatlist, 'DATE-OBS'))[0]
	Dateend = sorted(astro.get_headervalues(Flatlist, 'DATE-END'))[-1]
	try:
		del headerdict['EXPTIME']
	except KeyError:
		pass
	headerdict.update({'EXPOSURE':[Exp,'Total exposure (s)'], 'DATE-OBS':[Dateobs,''],'DATE-END':[Dateend,'']})
	astro.save_imagelist(flat_final, [headerdict], [Finalflat])
	print("flat final saved to", Finalflat)

	return flat_final

def process_flatframe_vbleexp(Flatlist, Darklist, Offsetlist, Finalflat, Config, Headers={}):
	"""
	---------------------
	Purpose
	Computes a master flat image from a series of several flat field, dark and offset FITS images, and saves it to a FITS file.
	This script assumes that a different integration time was used for each flat field frames, so possibly different from the (assumed constant) integration time used for the dark frames. To perform the dark current correction of the individual flats, it is necessary to have a series of offset frames.
	---------------------
	Inputs
	* Flatlist (list) = Flat images to be processed. Has the form [string_1, string_2, ..., string_n]. Each list element string_i is the complete filename (including path) of one FITS image.
	* Darklist (list) = Dark images to be processed. Has the form [string_1, string_2, ..., string_n]. Each list element string_i is the complete filename (including path) of one FITS image.
	* Offsetlist (list) = Offset images. Has the form [string_1, string_2, ..., string_n]. Each list element string_i is the complete filename (including path) of one FITS image.
	* Finalflat (list) = one element list containing a string [string]. It gives the complete filename (including path) of the FITS file where the master flat will be saved.
	* Config (integer) = the FITS headers of the final master flat image are copy-pasted from the first image in Flatlist. Config defines which set of rules must be used, additionally, to modify the FITS headers obtained from the image Flatlist[0]. The file conf.py allows to define several sets of rules and specifies which headers must be removed, kept, or added.
	* Headers (dictionary) = optional argument. Default value is an empty dictionary. Gives additional header entries, to be added to the headers of the final master flat image. If one keyword already exists in the headers of Flatlist[0], it will be overwritten by the new value/comment provided with Headers.
	FITS headers are created in three steps in this order:
		(1) headers are read-out from the image Flatlist[0] and modified according to the parameter Config,
		(2) additional header entries are added according to Headers,
		(3) as a last step, some modifications are done for the keywords EXPTIME, EXPOSURE, DATE-OBS and DATE-END: see Python code.
	---------------------
	Output (2D Numpy Array) = data of the final flat image.
	---------------------
	"""
	print("")
	print("aspylib.calib.process_flatframe_vbleexp():")
	
	#------ generating offset and dark ------
	offset_final = astro.add_median(astro.get_imagedata(Offsetlist))
	print("offset final generated ... ok")
	thermic_final = astro.add_median(astro.get_imagedata(Darklist)) - offset_final
	hotpixels = calib3.find_hotpixels(thermic_final, 100)	
	print("thermic final generated ... ok")

	#------ generating flat ------
	flats = astro.get_imagedata(Flatlist)
	Exp_dark = astro.get_headervalues(Darklist, 'EXPOSURE')[0]
	if Exp_dark=='':
		Exp_dark = astro.get_headervalues(Darklist, 'EXPTIME')[0]
	Exp_flat = astro.get_headervalues(Flatlist, 'EXPOSURE')
	if Exp_flat[0]=='':
		Exp_flat = astro.get_headervalues(Flatlist, 'EXPTIME')
	for i in range(len(Flatlist)):
		flats[i,:,:] = flats[i,:,:] - offset_final - thermic_final * Exp_flat[i]/Exp_dark
		flats[i,:,:] = flats[i,:,:] * 20000. / astro.get_median(flats[i,:,:])
	flat_final = astro.add_median(flats)
	del flats
	flat_avg = (np.roll(flat_final,1,axis=0) + np.roll(flat_final,-1,axis=0) + np.roll(flat_final,1,axis=1) + np.roll(flat_final,-1,axis=1)) / 4.
	flat_final = flat_avg*hotpixels + flat_final*(1.-hotpixels)			
	print("flat final generated ... ok")
	print("max value (ADU)=", np.max(flat_final.flatten()))

	#astro.display(flat_final, title='Final flat frame')

	#------ saving final image.fit ------
	headerdict = astro.create_headers(Config, [Flatlist[0]])[0]
	headerdict.update(Headers)
	if astro.get_headervalues([Flatlist[0]], 'EXPOSURE')[0]!='':
		Exp = np.sum(np.array(astro.get_headervalues(Flatlist, 'EXPOSURE')))
	else:
		Exp = np.sum(np.array(astro.get_headervalues(Flatlist, 'EXPTIME')))
	Dateobs = sorted(astro.get_headervalues(Flatlist, 'DATE-OBS'))[0]
	Dateend = sorted(astro.get_headervalues(Flatlist, 'DATE-END'))[-1]
	try:
		del headerdict['EXPTIME']
	except KeyError:
		pass
	headerdict.update({'EXPOSURE':[Exp,'Total exposure (s)'], 'DATE-OBS':[Dateobs,''],'DATE-END':[Dateend,'']})
	astro.save_imagelist(flat_final, [headerdict], [Finalflat])
	print("flat final saved to", Finalflat)

	return flat_final
