# -*- coding: iso_8859_1 -*-

import os, datetime, warnings
import numpy as np
from aspylib import astro

def write_auto_magnitudes(outputname, Imagelist, xy, sizes, dateobs, diff_mag, diff_stats, quality_flag, which_disk, ref_stars, datation='start', astrometry = []):
	"""
	---------------------
	Purpose
	This function writes the results obtained with photometry.compute_statistics() in a text file.
	---------------------
	Remark
	The following functions are meant to be used sequentially, with the outputs of the first function being used as inputs for the second one:
	1) photometry.compute_statistics(mag, nbref = 20, plotting = False) --> diff_mag, diff_stats, quality_flag, ref_stars, which_disk
	2) photometry.write_auto_magnitudes(outputname, Imagelist, xy, sizes, dateobs, diff_mag, diff_stats, quality_flag, which_disk, ref_stars, datation='start', astrometry = [])
	---------------------
	Inputs
	* outputname (string) = complete name (including path) of the text file where the information will be written. If the file already exists it will be overwritten.
	* Imagelist (list) = series of FITS images over which the photometry has been computed. Has the form [string_1, string_2, ..., string_i]. Each list element string_k is the complete filename (including path) of one FITS image.
	* xy (3D Numpy array) = array of the form [[[x11,y11], [x12,y12], ..., [x1N,y1N]],...,[[xi1,yi1], [xi2,yi2], ..., [xiN,yiN]]]. It gives the [x,y] coordinates for the N stars that have been photometrically measured over the i images. The variable xy[i] contains the coordinates of all stars on the ith image with filename Imagelist[i].
	* sizes (list) = list of the form [[center1, inner_radius1, outer_radius1], [center2, inner_radius2, outer_radius2], ..., [centerP, inner_radiusP, outer_radiusP]] where centeri, inner_radiusi and outer_radiusi are float numbers, which represent respectively 1) inneri = the radius of the disk within which the intensity of the star was summed up 2) inner_radiusi and 3) outer_radiusi are the inner and outer radii defining a disk with a hole, that is centered on the star, within which the sky background was evaluated. The P elements correspond to P measurements of the star magnitude which were done in parallel.
	* dateobs (list) = contains the Julian dates of each image. It can be the Julian date at the time of start of exposure, or mid-exposure, i.e. start_exposure + exposure/2 (see also datation).
	* diff_mag (2D Numpy Array) = array of the form diff_mag[i,j], provides the best differential magnitudes on image i for object j. Output of photometry.compute_statistics().
	* diff_stats (list) = list of 3 Numpy arrays [diff_signal, diff_noise_global, diff_noise_local]. The arrays diff_signal[j], diff_noise_global[j], diff_noise_local[j] are providing for star j respectively: the average magnitude, the RMS magnitude noise and the RMS magnitude noise after filtering out slow variations.  Output of photometry.compute_statistics().
	* quality_flag (list) = quality_flag[j] provides for each star j a flag indicating the quality of the measured differential magnitudes. Output of photometry.compute_statistics(). The possible flag values are:
		0 --> measurements are fine
		1 --> measurements are slightly degraded but still fine
		2 --> measurements are a bit more degraded
		3 --> measurements have average quality
		4 --> measurements have poor quality
		-1 --> measurements have not been accepted because quality is much too low
	* which_disk (list) = which_disk[j] gives the index (starting at 0 so 0,1,2,...) of the set of photometry disks that was used for the best differential magnitude of object j. Output of photometry.compute_statistics().
	* ref_stars (2D Numpy Array) = array of the form ref_stars[j,n]. For each set j of photometry disks, provides the list of nbref best reference stars (or less than nbref if there are not enough good reference stars), ordered by increasing noise. The list of best reference stars was calculated based on 2 criteria: (1) lowest noise (2) lowest peak to peak fluctuations of differential magnitude over the images. Output of photometry.compute_statistics().
	* datation (string) = optional argument. Defines whether if the date written in the text file is at the start (datation='start') or at the middle (datation='middle') of the exposure. Default value is 'start'. If the datation is at the start of exposure, the length of exposure in seconds is also written in the file.
	* astrometry (2D Numpy array) = optional argument. Default value is an empty list [] in which case no information on astrometry is written to the file. Otherwise, this input contains the following information about the matching of the measured stars with the USNO-B1 catalog, that will be written to the file:
		astrometry[0][i][0] (float) provides for the star i, the measured right ascension on the image, in degrees
		astrometry[0][i][1] (float) provides for the star i, the measured declination on the image, in degrees
		astrometry[1][i][0] (string) catalog name of USNO-B1 star matched with the star i
		astrometry[1][i][1] (float) provides the right ascension of the catalog star matched with star i, in degrees
		astrometry[1][i][2] (float) provides the declination of the catalog star matched with star i, in degrees
		astrometry[1][i][3] (float) provides the B1 magnitude of the catalog star matched with star i, in degrees
		astrometry[1][i][4] (float) provides the B2 magnitude of the catalog star matched with star i, in degrees
		astrometry[1][i][5] (float) provides the R1 magnitude of the catalog star matched with star i, in degrees
		astrometry[1][i][6] (float) provides the R2 magnitude of the catalog star matched with star i, in degrees
		Note: for the meaning of the B1,B2,R1 and R2 magnitudes, search online for information on the USNO-B1 catalog entries.
	---------------------
	Output = no output
	---------------------
	"""
	
	if os.path.exists(outputname):
		os.remove(outputname)
	f = open(outputname, 'w')

	#--- nb of stars to be written to file ---
	ind = list(np.where(np.array(quality_flag)>=0)[0])
	
	#--- file header ---
	f.write('#--- MAIN INFORMATION ------------------\n')
	f.write('date = '+str((datetime.datetime.now()).strftime("%Y-%m-%d %H:%M:%S"))+'\n')
	f.write(str(len(Imagelist))+' images processed\n')
	f.write(str(len(quality_flag))+' objects detected\n')
	f.write(str(len(ind))+' objects successfully measured\n')
	f.write('first image = '+(Imagelist[0])+'\n')
	if datation=='start':
		f.write('datation at start of exposure\n')
		h1 = astro.get_headervalues([Imagelist[0]],'EXPOSURE')[0]
		h2 = astro.get_headervalues([Imagelist[0]],'EXPTIME')[0]
		if h1!='':
			f.write('exposure time (sec):\t'+str(int(np.round(h1)))+'\n')
		elif h2!='':
			f.write('exposure time (sec):\t'+str(int(np.round(h2)))+'\n')
	elif datation=='middle':
		f.write('datation at middle of exposure\n')
	f.write('photometry with apertures:\n')
	for i in range(len(sizes)):	
		f.write('disk_'+str(i)+'\t'+str(sizes[i][0])+' - '+str(sizes[i][1])+' - '+str(sizes[i][2])+'\n')
	f.write('\n')

	#--- reference stars ---
	f.write('#--- REFERENCE STARS --\n')
	for i in range(len(sizes)):	
		f.write('disk_'+str(i)+':\n')
		f.write('nb of ref stars:' + str(len(ref_stars[i,:])) + '\n')
		f.write('ref stars:')
		for ref in np.sort(ref_stars[i,:]):
			f.write('\t' + str(ref+1))
		f.write('\n')
	f.write('\n')

	#--- stars X,Y coordinates ---
	f.write('#--- OBJECTS COORDINATES ON FIRST FRAME --\n')
	f.write('Object nb')
	for j in range(len(ind)):
		f.write('\t'+str(j+1))
	f.write('\n')
	f.write('X position (vertical)')
	for j in ind:
		f.write('\t'+"%.4f"%xy[0][j][0])
	f.write('\n')
	f.write('Y position (horizontal)')
	for j in ind:
		f.write('\t'+"%.4f"%xy[0][j][1])
	f.write('\n\n')

	#--- astrometry ---
	if len(astrometry)>0:
		positions = astrometry[0]
		f.write('#--- ASTROMETRY --\n')
		f.write('alpha')
		for j in ind:
			f.write('\t'+astro.decdegrees_to_hours(positions[j][0]))
		f.write('\n')
		f.write('delta')
		for j in ind:
			f.write('\t'+astro.decdegrees_to_degrees(positions[j][1]))
		f.write('\n\n')

	#--- catalog positions ---
	if len(astrometry)>0:
		usnob1 = astrometry[1]
		if len(usnob1)>0:
			f.write('#--- MATCHING WITH USNO-B1 --\n')
			f.write('distance (arcsec)')
			for j in ind:
				if len(usnob1[j])>2:
					alpha1 = positions[j][0]*np.pi/180.
					alpha2 = usnob1[j][1]*np.pi/180.
					delta1 = positions[j][1]*np.pi/180.
					delta2 = usnob1[j][2]*np.pi/180.
					dist = np.arccos(np.sin(delta1)*np.sin(delta2)+np.cos(delta1)*np.cos(delta2)*np.cos(alpha1-alpha2))*(180./np.pi)*3600.
					f.write('\t'+"%.2f"%dist)
				else:
					f.write('\t-')
			f.write('\n')
			f.write('name')
			for j in ind:
				if len(usnob1[j])>2:
					f.write('\t'+usnob1[j][0])
				else:
					f.write('\t-')
			f.write('\n')
			f.write('alpha')
			for j in ind:
				if len(usnob1[j])>2:
					f.write('\t'+astro.decdegrees_to_hours(usnob1[j][1]))
				else:
					f.write('\t-')
			f.write('\n')
			f.write('delta')
			for j in ind:
				if len(usnob1[j])>2:
					f.write('\t'+astro.decdegrees_to_degrees(usnob1[j][2]))
				else:
					f.write('\t-')
			f.write('\n')
			f.write('B1mag')
			for j in ind:
				if len(usnob1[j])>2:
					with warnings.catch_warnings():
						warnings.filterwarnings("ignore", category=UserWarning)		#to remove UserWarning when converting masked element to np.nan
						f.write('\t'+"%.2f"%usnob1[j][3])
				else:
					f.write('\t-')
			f.write('\n')
			f.write('B2mag')
			for j in ind:
				if len(usnob1[j])>2:
					with warnings.catch_warnings():
						warnings.filterwarnings("ignore", category=UserWarning)		#to remove UserWarning when converting masked element to np.nan
						f.write('\t'+"%.2f"%usnob1[j][4])
				else:
					f.write('\t-')
			f.write('\n')
			f.write('R1mag')
			for j in ind:
				if len(usnob1[j])>2:
					with warnings.catch_warnings():
						warnings.filterwarnings("ignore", category=UserWarning)		#to remove UserWarning when converting masked element to np.nan
						f.write('\t'+"%.2f"%usnob1[j][5])
				else:
					f.write('\t-')
			f.write('\n')
			f.write('R2mag')
			for j in ind:
				if len(usnob1[j])>2:
					with warnings.catch_warnings():
						warnings.filterwarnings("ignore", category=UserWarning)		#to remove UserWarning when converting masked element to np.nan
						f.write('\t'+"%.2f"%usnob1[j][6])
				else:
					f.write('\t-')
			f.write('\n\n')

	#--- statistics about photometry ---
	signal, noise_global, noise_local = diff_stats

	f.write('#--- STATISTICS ------------------------\n')
	f.write('mag signal')
	for j in ind:
		f.write('\t'+"%.6f"%signal[j])
	f.write('\n')
	f.write('global noise')
	for j in ind:
		f.write('\t'+"%.6f"%noise_global[j])
	f.write('\n')
	f.write('local noise')
	for j in ind:
		f.write('\t'+"%.6f"%noise_local[j])
	f.write('\n')
	f.write('quality flag')
	for j in ind:
		f.write('\t'+str(quality_flag[j]))
	f.write('\n')
	f.write('disk used for photometry')
	for j in ind:
		f.write('\t'+str(which_disk[j]))
	f.write('\n\n')
	
	#--- measured magnitudes ---
	f.write('#--- JULIAN DAY + MEASURED MAGNITUDES ---------------\n')
	data = np.ndarray((len(dateobs),len(ind)+1))
	data[:,0] = np.array(dateobs)
	data[:,1:] = np.ma.filled(diff_mag[:,ind],fill_value=np.nan)
	np.savetxt(f,data,delimiter='\t',fmt='%.6f')

	f.close()
	#-------------------------------------