# -*- coding: iso_8859_1 -*-

import time, sys
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages
from aspylib import astro, conf

def select_stars(Imagelist, shifts, pdfname, fit_area = 10, method = 'centroid'):
	"""
	---------------------
	Purpose
	This function allows to select targets (stars and/or moving objects) on a series of FITS image.
	The user selects the moving objects, variable stars and reference stars on the 1st image in the series and then the moving objects on the last image.
	The function then returns the positions of all moving objects, variable stars and reference stars on every image.
	There must be at least one moving object or variable star, and at least one reference star.
	---------------------
	Inputs
	* Imagelist (list) = series of FITS images where the stars will be selected. Has the form [string_1, string_2, ..., string_n]. Each list element string_i is the complete filename (including path) of one FITS image.
	* shifts (2D Numpy array) = shifts of all images with respect to the first one in the series. Has the shape (N,2) with N being the length of Imagelist2. Has the form [[x1,y1],[x2,y2],...,[xN,yN]] with x1=y1=0.
	* pdfname (string) = complete filename (including path) of the .pdf file where a screenshot of the 1st image is saved, with a matplotlib marker showing the positions of all selected moving objects, variable stars and reference stars.
	* fit_area (float) = optional argument, default value is 10. It defines the size of the square window where the maximum of signal is searched (if method='maximum' or 'centroid') and where a PSF fit is performed (if method='centroid'). If method='select' the variable fit_area is still used to compute 
	* method (string) = optional argument, default value is 'centroid'. It defines how the coordinates of the stars or moving objects are calculated on each image. Can be equal to 'select', 'maximum' or 'centroid'. If method='select' then rounded integer pixel positions are returned, that include the shifts of each image with respect to the first one as well as the movement of the object if it is moving. If method='maximum' then the position of the maximum signal within a square of size = 2*fit_area+1 centered on the position calculated including the image shift and object movement is returned. If method='centroid' then a PSF fit is performend within the window of size = 2*fit_area+1 centered on the position of the maximum. Therefore 'centroid' implements the maximum search followed by the PSF fit (1 additional step with respect to 'maximum').
	---------------------
	Outputs
	* names (list) = list of strings, each string has the form "moving_object_i", "vble_star_i" or "ref_star_i" with i=1,2,... and indicates the nature of each target.
	* xy (3D Numpy Array) = provides the position of each target on each image. Has the form [[[x1_1,y1_1],[x2_1,y2_1],...,[xP_1,yP_1]], ..., [[x1_N,y1_N],[x2_N,y2_N],...,[xP_N,yP_N]]] with P targets and N images. The targets are ordered in the same way as in names.
	---------------------
	"""
	print("")
	print("aspylib.photometry.select_stars():")
	
	#------------------------------------------------------------------------------------
	#	loading basic information
	#------------------------------------------------------------------------------------

	#------ initialisation of basic vbles ------
	xy_mov = []
	xy_vbl = []
	xy_ref = []
	
	#------ loading images ------
	image1 = astro.get_imagedata([Imagelist[0]], text_display=False)
	image2 = astro.get_imagedata([Imagelist[-1]], text_display=False)
	
	#------------------------------------------------------------------------------------
	#	selecting MOVING OBJECTS
	#------------------------------------------------------------------------------------

	print("")
	print('--- OBJECTS FOR photometry ------------------------------------------')
	print("")
	print('How many MOVING OBJECTS do you want to measure ?')
	astro.display(image1, title="First image")
	while 1:
		try:
			if (sys.version[0]=="2"):
				nb = int(raw_input('nb = '))
			elif (sys.version[0]=="3"):
				nb = int(input('nb = '))
			break
		except:
			print("please enter an integer !")

	if nb!=0:
		print("")
		print('First image : right-click on exact positions of the objects')
		xy1 = astro.select(image1, title='First image', nb=nb, color='red', edgecolor='red', marker='o')
		
		print("")
		print('Last image : right-click on exact positions of the objects (same order)')
		xy2 = astro.select(image2, title='Last image', nb=nb, color='red', edgecolor='red', marker='o')

		for i in range(len(xy1)):
			if (xy1[i][0]-fit_area<0)or(xy1[i][0]+fit_area+1>image1.shape[0])or(xy1[i][1]-fit_area<0)or(xy1[i][1]+fit_area+1>image1.shape[1]):
				print('moving object',i+1,'is too close from border of first image')
				print('this object is removed')
			elif (xy2[i][0]-fit_area<0)or(xy2[i][0]+fit_area+1>image1.shape[0])or(xy2[i][1]-fit_area<0)or(xy2[i][1]+fit_area+1>image1.shape[1]):
				print('moving object',i+1,'is too close from border of last image')
				print('this object is removed')
			else:
				xy_mov.append(xy1[i] + xy2[i])
				
	#------------------------------------------------------------------------------------
	#	selecting VARIABLE STARS
	#------------------------------------------------------------------------------------
	
	print("")
	print('How many VARIABLE STARS do you want to measure ?')
	fig, ax = astro.display(image1, title="First image",show=False)
	i=0
	for xy in xy_mov:
		i=i+1
		ax.scatter(xy[1],xy[0],s=50, marker='o', facecolor='none', edgecolor='red')
		ax.annotate(str(i), xy=(xy[1]+10, xy[0]+10),  xycoords='data', color='red') 
	plt.show()
	while 1:
		try:
			if (sys.version[0]=="2"):
				nb = int(raw_input('nb = '))
			elif (sys.version[0]=="3"):
				nb = int(input('nb = '))
			break
		except:
			print("please enter an integer !")
	
	if nb!=0:
		print("")
		print('First image : right-click on exact positions of stars')
		xy1, fig, ax = astro.select(image1, title='First image', nb=nb, color='magenta', edgecolor='magenta', marker='o', show=False)
		i=0
		for xy in xy_mov:
			i=i+1
			ax.scatter(xy[1],xy[0],s=50, marker='o', facecolor='none', edgecolor='red')
			ax.annotate(str(i), xy=(xy[1]+10, xy[0]+10),  xycoords='data', color='red') 
		plt.show()
		for i in range(len(xy1)):
			if (xy1[i][0]-fit_area<0)or(xy1[i][0]+fit_area+1>image1.shape[0])or(xy1[i][1]-fit_area<0)or(xy1[i][1]+fit_area+1>image1.shape[1]):
				print('variable star',i+1,'is too close from border of first image')
				print('this star is removed')
			else:
				xy_vbl.append(xy1[i])

	#------------------------------------------------------------------------------------
	#	selecting REFERENCE STARS
	#------------------------------------------------------------------------------------

	print("")
	print("")
	print('--- REFERENCE STARS -------------------------------------------------')
	print("")
	print('How many REFERENCE STARS do you want to measure ?')
	fig, ax = astro.display(image1, title="First image",show=False)
	i=0
	for xy in xy_mov:
		i=i+1
		ax.scatter(xy[1],xy[0],s=50, marker='o', facecolor='none', edgecolor='red')
		ax.annotate(str(i), xy=(xy[1]+10, xy[0]+10),  xycoords='data', color='red') 
	i=0
	for xy in xy_vbl:
		i=i+1
		ax.scatter(xy[1],xy[0],s=50, marker='o', facecolor='none', edgecolor='magenta')
		ax.annotate(str(i), xy=(xy[1]+10, xy[0]+10),  xycoords='data', color='magenta') 
	plt.show()
	while 1:
		try:
			if (sys.version[0]=="2"):
				nb = int(raw_input('nb = '))
			elif (sys.version[0]=="3"):
				nb = int(input('nb = '))
			break
		except:
			print("please enter an integer !")
	
	if nb!=0:
		print("")
		print('First image : right-click on exact positions of stars')
		xy1, fig, ax = astro.select(image1, title='First image', nb=nb, color='green', edgecolor='green', marker='o', show=False)
		i=0
		for xy in xy_mov:
			i=i+1
			ax.scatter(xy[1],xy[0],s=50, marker='o', facecolor='none', edgecolor='red')
			ax.annotate(str(i), xy=(xy[1]+10, xy[0]+10),  xycoords='data', color='red') 
		i=0
		for xy in xy_vbl:
			i=i+1
			ax.scatter(xy[1],xy[0],s=50, marker='o', facecolor='none', edgecolor='magenta')
			ax.annotate(str(i), xy=(xy[1]+10, xy[0]+10),  xycoords='data', color='magenta') 
		plt.show()
		for i in range(len(xy1)):
			if (xy1[i][0]-fit_area<0)or(xy1[i][0]+fit_area+1>image1.shape[0])or(xy1[i][1]-fit_area<0)or(xy1[i][1]+fit_area+1>image1.shape[1]):
				print('reference star',i+1,'is too close from border of first image')
				print('this star is removed')
			else:
				xy_ref.append(xy1[i])
	
	#------------------------------------------------------------------------------------
	#	saving selected stars in .pdf
	#------------------------------------------------------------------------------------

	fig, ax = astro.display(image1, title="First image", show=False)
	i=0
	for xy in xy_mov:
		i=i+1
		ax.scatter(xy[1],xy[0],s=50, marker='o', facecolor='none', edgecolor='red')
		ax.annotate(str(i), xy=(xy[1]+10, xy[0]+10),  xycoords='data', color='red') 
	i=0
	for xy in xy_vbl:
		i=i+1
		ax.scatter(xy[1],xy[0],s=50, marker='o', facecolor='none', edgecolor='magenta')
		ax.annotate(str(i), xy=(xy[1]+10, xy[0]+10),  xycoords='data', color='magenta') 
	i=0
	for xy in xy_ref:
		i=i+1
		ax.scatter(xy[1],xy[0],s=50, marker='o', facecolor='none', edgecolor='green')
		ax.annotate(str(i), xy=(xy[1]+10, xy[0]+10),  xycoords='data', color='green') 
	pp = PdfPages(pdfname)
	pp.savefig()
	pp.close()
	plt.close()
	
	#------------------------------------------------------------------------------------
	#	check that one object + one ref star at least have been selected
	#------------------------------------------------------------------------------------

	nb_obj = len(xy_mov)+len(xy_vbl)
	nb_ref = len(xy_ref)

	if nb_obj==0:
		print("")
		input('no object has been selected, press ENTER to close window')
		sys.exit()

	if nb_ref==0:
		print("")
		input('no reference star has been selected, press ENTER to close window')
		sys.exit()

	#------------------------------------------------------------------------------------
	#	loading basic information (2)
	#------------------------------------------------------------------------------------

	#--- initialising main variables ---
	time_format = "%Y-%m-%dT%H:%M:%S"
	xy=[]		#X-Y positions on 1st image
	dxy=[]		#X-Y positions change per unit time
	names = []

	#------ first image ------
	date1 = astro.get_headervalues([Imagelist[0]], 'DATE-OBS')
	t1 = time.mktime(time.strptime(date1[0].split('.')[0],time_format))
	
	#------ last image ------
	date2 = astro.get_headervalues([Imagelist[-1]], 'DATE-OBS')
	t2 = time.mktime(time.strptime(date2[0].split('.')[0],time_format))

	#------------------------------------------------------------------------------------
	#	calculating approximate coordinates on each frame
	#------------------------------------------------------------------------------------	
	
	if len(xy_mov)!=0:		
		xs, ys = shifts[-1]		#shift between first and last images

		for i in range(len(xy_mov)):
				
			#--- collecting information ---
			xy.append([np.float(xy_mov[i][0]), np.float(xy_mov[i][1])])
			dxy.append([np.float(xy_mov[i][2]-xy_mov[i][0]-xs)/(t2-t1), np.float(xy_mov[i][3]-xy_mov[i][1]-ys)/(t2-t1)])
			names.append('moving_object_'+str(i+1))

	if len(xy_vbl)!=0:		
		for i in range(len(xy_vbl)):
			xy.append([np.float(xy_vbl[i][0]), np.float(xy_vbl[i][1])])
			dxy.append([0., 0.])
			names.append('vble_star_'+str(i+1))

	if len(xy_ref)!=0:		
		for i in range(len(xy_ref)):
			xy.append([np.float(xy_ref[i][0]), np.float(xy_ref[i][1])])
			dxy.append([0., 0.])
			names.append('ref_star_'+str(i+1))

	xy = np.array(xy)
	dxy = np.array(dxy)
			
	#------------------------------------------------------------------------------------
	#	calculating positions of stars on all frames
	#------------------------------------------------------------------------------------

	if method=='select':
		print("")
		print('--- CALCULATING STAR POSITIONS --------------------------------------')
		print("")

		xy_sel = []
		for i in range(len(Imagelist)):					#loop on images
		
			print('calculating star position on :',Imagelist[i].split(conf.sep)[-1])
			if i==0:
				image = image1
			else:		
				image = astro.get_imagedata([Imagelist[i]], text_display=False)

			date = astro.get_headervalues([Imagelist[i]], 'DATE-OBS')
			t = time.mktime(time.strptime(date[0].split('.')[0],time_format))		
			xyi = np.round(xy+shifts[i]+(t-t1)*dxy)

			xy_sel.append(xyi)
		xy = np.array(xy_sel)
	
	elif method=='maximum':
		print("")
		print('--- CALCULATING LOCAL MAX FOR STAR POSITION -------------------------')
		print("")

		xy_max = []
		for i in range(len(Imagelist)):					#loop on images
		
			print('calculating local maxima on :',Imagelist[i].split(conf.sep)[-1])
			if i==0:
				image = image1
			else:		
				image = astro.get_imagedata([Imagelist[i]], text_display=False)

			date = astro.get_headervalues([Imagelist[i]], 'DATE-OBS')
			t = time.mktime(time.strptime(date[0].split('.')[0],time_format))		
			xyi = np.round(xy+shifts[i]+(t-t1)*dxy)

			xy_image = astro.max_stars(image, xyi, fit_area, error_messages = 2)			
			xy_max.append(xy_image)
		xy = np.array(xy_max)
		
	elif method=='centroid':
		print("")
		print('--- CALCULATING STAR CENTROID WITH PSF FIT --------------------------')
		print("")

		xy_max = []
		for i in range(len(Imagelist)):					#loop on images
		
			print('calculating local maxima on :',Imagelist[i].split(conf.sep)[-1])
			if i==0:
				image = image1
			else:		
				image = astro.get_imagedata([Imagelist[i]], text_display=False)

			date = astro.get_headervalues([Imagelist[i]], 'DATE-OBS')
			t = time.mktime(time.strptime(date[0].split('.')[0],time_format))		
			xyi = np.round(xy+shifts[i]+(t-t1)*dxy)

			xy_image = astro.max_stars(image, np.array(xyi,dtype=np.int), fit_area, error_messages = 2)			
			xy_max.append(xy_image)
		xy_max = np.array(xy_max)
		#-----------------------------------------------------------------------
		xy_psf = []
		print("")
		for i in range(len(Imagelist)):					#loop on images
			
			print('PSF fitting on :',Imagelist[i].split(conf.sep)[-1])
			if i==0:
				image = image1
			else:
				image = astro.get_imagedata([Imagelist[i]], text_display=False)

			xy_image = astro.fit_stars(image, xy_max[i], fit_area, maxshift = fit_area*0.5, error_messages = 2)
		
			xy_psf.append(xy_image)
		xy = np.array(xy_psf)
			
		return names, xy
