# -*- coding: iso_8859_1 -*-

import sys
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.font_manager as fm
from aspylib import conf

def check_refstars(names, mag):
	"""
	---------------------
	Purpose
	Given the measured magnitudes for a set of N targets (moving objects, variable stars or moving objects), over i images and with P sets of photometry disks, this function detects and removes the data for the reference stars and images that have a poor quality.
	After this function has been applied, the set of reference stars does not contain any wrong measurement (i.e. all magnitudes equal to np.nan have been masked). Additionally the user can control the quality of the reference stars data visually on a matplotlib plot and remove the points that he considers not good enough.
	---------------------
	Remark
	The following functions are meant to be used sequentially, with the outputs of a function being used as inputs for the next ones:
	1) photometry.check_refstars(names, mag) --> mag_ok, rejstar, rejdata
	2) photometry.filter_magnitudes(obj_index, names, sizes, mag_ok, rejdata) --> mag_i, rejdata_i, rejdisk_i
	3) photometry.best_magnitude(obj_index, names, sizes, mag_i, rejstar, rejdata_i, rejdisk_i, nb=10) --> ref_1, mag_1, sig_1
	4) photometry.write_best_magnitude(outputname, Imagelist, names, obj_index, sizes, dateobs, ref_1, mag_1, sig_1, rejdata_i, rejsizes=[], rejstar=[], nb=10, datation='start')
	OR
	photometry.write_cdrcdl_report(outputname, Imagelist, names, obj_index, dateobs, mag_i, ref_1, mag_1, sig_1, rejdata_i, format='DVR')
	---------------------
	Inputs
	* names (list) = list of strings, each string has the form "moving_object_i", "vble_star_i" or "ref_star_i" with i=1,2,... and indicates the nature of each target.
	* mag (3D Numpy array) = measured magnitudes. Has the form [[[m1_1_1, m1_1_2, ..., m1_1_P], [m1_2_1, m1_2_2, ..., m1_2_P], ..., [m1_N_1, M1_N_2, ..., M1_N_P]], ..., [[mi_1_1, mi_1_2, ..., mi_1_P], [mi_2_1, mi_2_2, ..., mi_2_P], ..., [mi_N_1, Mi_N_2, ..., Mi_N_P]]] where P=len(sizes) is the number of sets of photometry disks provided as input, N=len(coord) is the number of stars and i=len(Imagelist) the number of images. 
	---------------------
	Outputs
	* mag_ok (3D Numpy masked array) = same as mag, with a mask set equal to False for some images and reference stars which do not have good enough quality.
	* rejstar (list) = indices (starting at 0 = 0,1,2,3,...) of the rejected reference stars.
	* rejdata (list) = indices (starting at 0 = 0,1,2,3,...) of the rejected images.
	---------------------
	"""
	print("")
	print("aspylib.photometry.check_refstars():")
	
	nb_obj = len([i for i in names if (i[0]=='m')or(i[0]=='v')])
	nb_ref = len([i for i in names if (i[0]=='r')])

	rejstar = []
	rejdata = []

	mask = np.ma.make_mask_none(np.shape(mag))
	mag_ok = np.ma.masked_array(mag,mask=mask)

	#------ first check to reject wrong measurements --------------

	#removing reference star that are poorly measured
	for i in range(nb_obj,len(names)):		#loop on all reference stars
		test_refstar = np.sum(np.isnan(mag_ok[:,i,:]).flatten())
		total = np.shape(mag_ok)[0] * np.shape(mag_ok)[2]
		if test_refstar >= 0.05*total:
			print("Object " + names[i] + " has more than 5% of wrong measurements, it is removed")
			rejstar.append(i)
			mag_ok.mask[:,i,:]=True		

	#removing image for which remaining reference stars are not correctly measured
	for i in range(len(mag_ok)):		#loop on all images
		test_images = np.sum(np.isnan(mag_ok[i,nb_obj:,:]).flatten())
		if test_images > 0:
			print("Image " + str(i+1) + " has wrong measurements, it is removed")
			rejdata.append(i)
			mag_ok.mask[i,:,:] = True

	#--> at this moment we have a set of images and reference stars free of any wrong measurement

	#------ second check based on user visual verification --------------
	if (nb_ref-len(rejstar)>1):	#the ref star check can be continued		

		while True:
			#------ plot instrument differential magnitudes for reference stars ------
			fig = plt.figure()
			fig.canvas.set_window_title("Reference stars")
			plt.axes([0.1,0.1,0.7,0.8])

			indices = ~(mag_ok.mask[:,0,0])	#indexing unmasked images
			for j in range(nb_obj,len(names)):
				if j not in rejstar:
					plt.plot(mag_ok[indices,j,0] - np.mean(mag_ok[:,nb_obj:,0], axis=1)[indices], conf.col[(j-nb_obj)%16], linewidth=1.3, label=(names[j]))

			plt.legend(prop=fm.FontProperties(size=11), bbox_to_anchor=(1.02, 1.02), loc=2)
			plt.title("Differential magnitudes for REF stars")
			plt.grid()
			ax = plt.gca()
			ax.set_ylim(ax.get_ylim()[::-1])	
			print("")
			print("Are reference stars OK (answer must be 'yes' or 'no')?")
			plt.show()
			
			#------ question ---------------------------------------------------------
			if (sys.version[0]=="2"):
				ans = raw_input()
			elif (sys.version[0]=="3"):
				ans = input()

			if (ans=='yes'):
				break
			else:
				#------ then plot instrumental magnitudes ----------------------------
				fig = plt.figure()
				fig.canvas.set_window_title("Reference stars")
				plt.axes([0.1,0.1,0.7,0.8])
			
				indices = ~(mag_ok.mask[:,0,0])	#indexing unmasked images
				for j in range(nb_obj,len(names)):
					if j not in rejstar:
						plt.plot(mag_ok[indices,j,0], conf.col[(j-nb_obj)%16], linewidth=1.3, label=(names[j]))

				plt.legend(prop=fm.FontProperties(size=11), bbox_to_anchor=(1.02, 1.02), loc=2)
				plt.grid()
				plt.title("Instrumental magnitudes for REF stars")
				ax = plt.gca()
				ax.set_ylim(ax.get_ylim()[::-1])				

				print("")
				print("which reference star is wrong ?")
				plt.show()

				while True:
					try:
						if (sys.version[0]=="2"):
							wrong = np.int(raw_input())
						elif (sys.version[0]=="3"):
							wrong = np.int(input())
						break
					except:
						print("please enter an integer !")

				if (wrong>=1)and(wrong<=nb_ref):
					if (nb_obj+wrong-1) not in rejstar:
						rejstar.append(nb_obj+wrong-1)
						mag_ok.mask[:,nb_obj+wrong-1,:] = True

	return mag_ok, rejstar, rejdata