# -*- coding: iso_8859_1 -*-

import datetime, sys
import os
import numpy as np
import matplotlib.pyplot as plt
from aspylib import astro, conf

def check_pictures_quality(outputname, Imagelist):
	"""
	---------------------
	Purpose
	Check the radiometric quality of a series of FITS images, identify the images that have a weak signal (e.g. due to fog) or a non-uniform background (e.g. due to a cloud passing through the field of view, or to some straylight) and flag them so that they won't be use for extracting the photometry measurements.
	A file is created where all the information about radiometric quality, and the flagged images, is saved.
	---------------------
	Inputs
	* outputname (string) = complete filename (including path) of the text file where the information about radiometric quality and the flagged images will be stored.
	* Imagelist (list) = series of FITS images to be checked. Has the form [string_1, string_2, ..., string_n]. Each list element string_i is the complete filename (including path) of one FITS image.
	---------------------
	Outputs
	* good_Imagelist (list) = FITS images that have passed succesfully the test and will be used for photometry measurements. Has the form [string_1, string_2, ..., string_n]. Each list element string_i is the complete filename (including path) of one FITS image.
	---------------------
	"""
	print("")
	print("aspylib.photometry.check_pictures_quality():")
	
	ind = list(np.array(np.arange(1,len(Imagelist)+1), dtype=np.int))
	
	#--- criteria ---
	maxi = []
	mini = []
	globmaxi = []
	globmini = []
	median1 = []
	median2 = []
	median3 = []
	median4 = []
	median5 = []

	#--- rejected ---
	rej1 = []
	rej2 = []
	rej3 = []
	
	print("")
	print('--- CALCULATING STATISTICS ------------------------------------------')

	#--- looping on images ---
	for i in range(len(Imagelist)):
		if i==0:
			print("")
			print('You have the possibility to check the min and max signals of the images, and the uniformity of the image background. This step allows to reject images acquired in bad conditions (clouds passing, obstruction by trees, straylight, etc).')
			print("")
			print('There are 2 possibilities')
			print('(0) all images are ok, no check is needed')
			print('(1) a check is needed')
			print("")
			print('What is your choice (0 or 1) ?')
			image0 = astro.get_imagedata([Imagelist[i]], text_display=False)
			astro.display(image0, title="First image")
			while 1:
				try:
					if (sys.version[0]=="2"):
						ans = int(raw_input())
					elif (sys.version[0]=="3"):
						ans = int(input())
					break
				except:
					print("please enter an integer !")

			if ans ==1:
				#--- calculating combined image ---
				dN = int(len(Imagelist)/4)
				N = [0,dN,dN*2,dN*3,len(Imagelist)-1]
				imagedrift = np.array(image0)
				for i in range(4):
					imagedrift += astro.get_imagedata([Imagelist[N[i]]], text_display=False)
				imagedrift /= 5
				#--- done -------------------------
				print("")
				print('5 images taken at regular intervals are added and displayed to give an estimation of the image drift.')
				print('Right-click to select the 4 corners of some part of this combined image. The selected area will be used to check the variations of the min and max signals.')
				xy = astro.select(imagedrift, nb=4, facecolor = 'red', marker= '+', title="Sum of 5 images over the series")
				included = [min([xy[0][0],xy[1][0],xy[2][0],xy[3][0]]), \
							max([xy[0][0],xy[1][0],xy[2][0],xy[3][0]]), \
							min([xy[0][1],xy[1][1],xy[2][1],xy[3][1]]), \
							max([xy[0][1],xy[1][1],xy[2][1],xy[3][1]])] 
			else:
				return Imagelist
		else:
			image0 = astro.get_imagedata([Imagelist[i]])

		#--- calculating dynamics ---
		image1 = image0
		if len(included)!=0:
			image1 = image1[included[0]:included[1],included[2]:included[3]]

		maxi.append(np.max(image1.flatten()))
		mini.append(np.min(image1.flatten()))
		globmaxi.append(np.max(image0.flatten()))
		globmini.append(np.min(image0.flatten()))
		
		#--- calculating background uniformity ---
		sizeX = np.shape(image0)[0]
		sizeY = np.shape(image0)[1]
		m1 = np.median(image0[int(sizeX/2)-100:int(sizeX/2)+100,int(sizeY/2)-100:int(sizeY/2)+100].flatten())
		m2 = np.median(image0[100:200,100:200].flatten())
		m3 = np.median(image0[sizeX-200:sizeX-100,100:200].flatten())
		m4 = np.median(image0[100:200,sizeY-200:sizeY-100].flatten())
		m5 = np.median(image0[sizeX-200:sizeX-100,sizeY-200:sizeY-100].flatten())
		moy = (m1+m2+m3+m4+m5)/5.

		#background non-uniformity (degrades photometry for moving target)
		median1.append(m1-moy)
		median2.append(m2-moy)
		median3.append(m3-moy)
		median4.append(m4-moy)
		median5.append(m5-moy)
			
	#variations of background non-uniformity (degrades photometry for fixed target)
	varmedian1 = median1 - np.mean(median1)
	varmedian2 = median2 - np.mean(median2)
	varmedian3 = median3 - np.mean(median3)
	varmedian4 = median4 - np.mean(median4)
	varmedian5 = median5 - np.mean(median5)

	#calculating dynamic range
	dyn = np.array(maxi) - np.array(mini)
	globdyn = np.array(globmaxi) - np.array(globmini)

	print("")
	print("")
	print('--- CHECKING DYNAMIC RANGE OF CALIBRATED IMAGES ---------------------')
	
	fig = plt.figure(figsize=(8,10))
	fig.canvas.set_window_title("Dynamic range") 
	ax = fig.add_subplot(2,1,1)
	ax.plot(ind,globmaxi,label='Max')
	ax.plot(ind,globmini,label='Min')
	ax.plot(ind,globdyn,label='Dynamic range')
	ax.set_title("Min, Max, Max-Min (on full image)")
	ax.set_xlabel("image number")
	ax.grid()
	ax = fig.add_subplot(2,1,2)
	ax.plot(ind,maxi,label='Max')
	ax.plot(ind,mini,label='Min')
	ax.plot(ind,dyn,label='Dynamic range')
	ax.set_title("Min, Max, Max-Min (on selected sub-part)")
	ax.set_xlabel("image number")
	ax.grid()
	print("Now, please chose a threshold value. All images having the difference between max and min (=dynamic range) below this value will be rejected (test performed on the second plot).")
	print("Enter the value:")
	plt.show()
	while 1:
		try:
			if (sys.version[0]=="2"):
				th1 = np.float(raw_input())
			elif (sys.version[0]=="3"):
				th1 = np.float(input())
			break
		except:
			print("please enter a number !")
	for i,m in enumerate(dyn):
		if m>=th1:
			rej1.append(i)
	rej1.sort()

	xx = []
	print("")
	for yy in rej1:
		xx.append(yy+1)
	print("Images to be removed:",xx)

	print("")
	print("")
	print('--- CHECKING UNIFORMITY OF CALIBRATED IMAGES -----------------------')
			
	fig = plt.figure(figsize=(8,10))
	fig.canvas.set_window_title("Background uniformity") 
	ax = fig.add_subplot(2,1,1)
	ax.plot(ind,median1)
	ax.plot(ind,median2)
	ax.plot(ind,median3)
	ax.plot(ind,median4)
	ax.plot(ind,median5)
	ax.set_title("Deviation from uniform sky background\n(measured at center + 4 corners)")
	ax.set_xlabel("image number")
	ax.grid()
	ax = fig.add_subplot(2,1,2)
	ax.plot(ind,varmedian1)
	ax.plot(ind,varmedian2)
	ax.plot(ind,varmedian3)
	ax.plot(ind,varmedian4)
	ax.plot(ind,varmedian5)
	ax.set_title("Variations of the above plot")
	ax.set_xlabel("image number")
	ax.grid()	
	print("Now, please chose a threshold value. All images with background non-uniformity higher than this value (on the second plot) are rejected.")
	print("Enter the value:")
	plt.show()

	while 1:
		try:
			if (sys.version[0]=="2"):
				th2 = np.float(raw_input())
			elif (sys.version[0]=="3"):
				th2 = np.float(input())
			break
		except:
			print("please enter a number !")
	for i,m in enumerate(varmedian1):
		if np.abs(m)>=th2:
			rej2.append(i)

	for i,m in enumerate(varmedian2):
		if np.abs(m)>=th2:
			rej2.append(i)

	for i,m in enumerate(varmedian3):
		if np.abs(m)>=th2:
			rej2.append(i)

	for i,m in enumerate(varmedian4):
		if np.abs(m)>=th2:
			rej2.append(i)

	for i,m in enumerate(varmedian5):
		if np.abs(m)>=th2:
			rej2.append(i)

	rej2 = list(set(rej2))
	rej2.sort()
	plt.close()
	
	xx = []
	print("")
	for yy in rej2:
		xx.append(yy+1)
	print("Images to be removed:",xx)
		
	print("")
	print("")
	print('--- ADDITIONAL IMAGES TO BE REMOVED ---------------------------------')
	while 1:
		try:
			if (sys.version[0]=="2"):
				im = int(raw_input("Please enter the numbers of additional images to be removed:"))
			elif (sys.version[0]=="3"):
				im = int(input("Please enter the numbers of additional images to be removed:"))
			rej3.append(im-1)
		except:
			break
	rej3 = list(set(rej3))
	rej3.sort()
	
	#------ all removed images ------------------------------------
	rejected = rej1+rej2+rej3
	rejected = list(set(rejected))
	rejected.sort()

	print("")
	print("")
	print('--- FINAL LIST OF IMAGES TO BE REMOVED ------------------------------')
	xx = []
	for yy in rejected:
		xx.append(yy+1)
	print(xx)

	good_Imagelist = [Imagelist[i] for i in range(len(Imagelist)) if i not in rejected]

	#------ writing results to output file ---------------------------------
	#names of output files
	if os.path.exists(outputname):
		os.remove(outputname)

	f = open(outputname, 'w')
	f.write('date:\t'+str((datetime.datetime.now()).strftime("%Y-%m-%d %H:%M:%S"))+'\n')
	f.write("Working directory:\n"+os.path.split(Imagelist[0])[0]+"\n\n")
	f.write("-------------------------------------------------\n")
	f.write("      User inputs for images selection\n")
	f.write("-------------------------------------------------\n")
	f.write("Dynamic range threshold:\t"+str(th1)+"\n")
	f.write("Background uniformity threshold:\t"+str(th2)+"\n")
	f.write("Images rejected by user:\t")
	if len(rej3)==0:
		f.write('none\n')
	else:
		for i in range(len(rej3)-1):
			f.write(str(rej3[i]+1)+',')
		f.write(str(rej3[-1]+1)+"\n")
	f.write("\n")
	
	f.write("-------------------------------------------------\n")
	f.write("      Input images are\n")
	f.write("-------------------------------------------------\n")
	f.write('Image\tImage status\tImage name\tGlobal min value\tGlobal max value\tMin value\tMax value\tSky background 1\tSky background 2\tSky background 3\tSky background 4\tSky background 5\tVariation sky background 1\tVariation sky background 2\tVariation sky background 3\tVariation sky background 4\tVariation sky background 5\n')
	for i,im in enumerate(Imagelist):
		f.write(str(i+1)+'\t')
		if i not in rejected:
			f.write("ok\t")
		else:
			f.write("rejected (")
			test = (i in rej1)*100 + (i in rej2)*10 + (i in rej3)
			if test == 100:
				f.write("dyn)\t")
			elif test == 10:
				f.write("bkg)\t")
			elif test == 1:
				f.write("usr)\t")
			elif test == 11:
				f.write("bkg+usr)\t")
			elif test == 101:
				f.write("dyn+usr)\t")
			elif test == 110:
				f.write("dyn+bkg)\t")
			elif test == 111:
				f.write("dyn+bkg+usr)\t")
		f.write(im.split(conf.sep)[-1]+'\t')
		f.write(str(globmini[i])+'\t'+str(globmaxi[i])+'\t'+str(mini[i])+'\t'+str(maxi[i])+'\t')
		f.write("%.3f"%median1[i]+'\t'+"%.3f"%median2[i]+'\t'+"%.3f"%median3[i]+'\t'+"%.3f"%median4[i]+'\t'+"%.3f"%median5[i]+'\t')
		f.write("%.3f"%varmedian1[i]+'\t'+"%.3f"%varmedian2[i]+'\t'+"%.3f"%varmedian3[i]+'\t'+"%.3f"%varmedian4[i]+'\t'+"%.3f"%varmedian5[i]+'\n')
	f.close()

	return good_Imagelist