# -*- coding: iso_8859_1 -*-

#import sys
import numpy as np
from aspylib import astro, conf

def make_photometry(Imagelist, sizes, coord, method='centroid', check_sat = [True,'saturation'], mag_sat = np.nan, mag_zero = np.nan, error_messages = 1):
	"""
	---------------------
	Purpose
	Wrapper function for calling aperture_photometry() (which works for only one image) over as series of images.
	Given the coordinates of N stars on i images (3D array with 2*N*i values) and a list of P photometry disks, the function returns N*i*P magnitude values (3D array).
	---------------------
	Inputs
	* Imagelist (list) = series of FITS images over which the photometry must be measured. Has the form [string_1, string_2, ..., string_i]. Each list element string_k is the complete filename (including path) of one FITS image.
	* sizes (list) = list of the form [[center1, inner_radius1, outer_radius1], [center2, inner_radius2, outer_radius2], ..., [centerP, inner_radiusP, outer_radiusP]] where centeri, inner_radiusi and outer_radiusi are float numbers, which represent respectively 1) inneri = the radius of the disk within which the intensity of the star is summed up 2) inner_radiusi and 3) outer_radiusi are the inner and outer radii defining a disk with a hole, that is centered on the star, within which the sky background is evaluated. The P elements correspond to P measurements of the star magnitude which are done in parallel.
	* coord (3D Numpy array) = array of the form [[[x11,y11], [x12,y12], ..., [x1N,y1N]],...,[[xi1,yi1], [xi2,yi2], ..., [xiN,yiN]]]. It gives the [x,y] coordinates for the N stars to be photometrically measured over the i images. The variable coord[i] contains the coordinates of all stars on the ith image with filename Imagelist[i].
	* method (string) = optional argument, default value is 'centroid'. Defines the method used to compute the magnitude of a star. If equal to 'centroid' then the central photometry disk is centered accurately on the non-integer coordinates of the provided star centroid. If equal to any other value, then the disk is centered on rounded integer coordinates. The 'centroid' method has been tested extensively and gives more accurate results, with lower noise and systematic errors.
	* check_sat (list) = optional argument, default value is a two elements list [True,'saturation']. This parameter defines how saturated stars are handled. It is only used if check_sat[0] = True. Can be either equal to [True,'saturation'] or to a two elements list [True,XXX] where XXX is a float number. The float number gives the value above which it is assumed that a pixel is saturated. If check_sat[1]='saturation' then the value conf.saturation is taken.
	* mag_sat (float) = optional argument, default value is np.nan. Magnitude value returned by the function in case a star is saturated.
	* mag_zero (float) = optional argument, default value is np.nan. Magnitude value returned by the function in case a star has an intensity below the sky background.
	* error_messages (integer) = optional argument. It defines the level of information displayed in the console window about photometry errors. Default value is 1, can be equal to 0 (no information displayed), 1 (basic information) or 2 (more detailed information).
	---------------------
	Outputs
	* mag_total (3D Numpy array) = returned measured magnitudes. Has the form [[[m1_1_1, m1_1_2, ..., m1_1_P], [m1_2_1, m1_2_2, ..., m1_2_P], ..., [m1_N_1, M1_N_2, ..., M1_N_P]], ..., [[mi_1_1, mi_1_2, ..., mi_1_P], [mi_2_1, mi_2_2, ..., mi_2_P], ..., [mi_N_1, Mi_N_2, ..., Mi_N_P]]] where P=len(sizes) is the number of sets of photometry disks provided as input, N=len(coord) is the number of stars and i=len(Imagelist) the number of images. 
	---------------------
	"""
	print("")
	print("aspylib.photometry.make_photometry():")

	#loop on images
	print("")
	mag_total = []
	
	for i in range(len(Imagelist)):
		#loads relevant image
		print('Photometry on :',Imagelist[i].split(conf.sep)[-1])
		image = astro.get_imagedata([Imagelist[i]], text_display=False)

		mag_total.append(aperture_photometry(image, sizes, coord[i], method=method, check_sat = check_sat, mag_sat = mag_sat, mag_zero = mag_zero, error_messages = error_messages))
	
	mag_total = np.array(mag_total)
	return mag_total

def aperture_photometry(image, sizes, coord, method='centroid', check_sat = [True,'saturation'], mag_sat = np.nan, mag_zero = np.nan, error_messages = 1):
	"""
	---------------------
	Purpose
	Computes the magnitudes of a list of N stars over a FITS image, given their (x,y) coordinates on that particular image, and the size of the photometry disks that must be used.
	The function allows to applies an arbitrary number P of photometry disks, so that the user can inspect the results and keep the ones with the best photometric quality.
	The function returns N*P magnitude values.
	---------------------
	Inputs
	* image (2D Numpy array) = data of the radiometrically calibrated image on which photometry will be performed.
	* sizes (list) = list of the form [[center1, inner_radius1, outer_radius1], [center2, inner_radius2, outer_radius2], ..., [centerP, inner_radiusP, outer_radiusP]] where centeri, inner_radiusi and outer_radiusi are float numbers, which represent respectively 1) inneri = the radius of the disk within which the intensity of the star is summed up 2) inner_radiusi and 3) outer_radiusi are the inner and outer radii defining a disk with a hole, that is centered on the star, within which the sky background is evaluated. The P elements correspond to P measurements of the star magnitude which are done in parallel.
	* coord (2D Numpy array) = array of the form [[x1,y1], [x2,y2], ..., [xN,yN]]. It gives the [x,y] coordinates for the N stars to be photometrically measured.
	* method (string) = optional argument, default value is 'centroid'. Defines the method used to compute the magnitude of a star. If equal to 'centroid' then the central photometry disk is centered accurately on the non-integer coordinates of the provided star centroid. If equal to any other value, then the disk is centered on rounded integer coordinates. The 'centroid' method has been tested extensively and gives more accurate results, with lower noise and systematic errors.
	* check_sat (list) = optional argument, default value is a two elements list [True,'saturation']. This parameter defines how saturated stars are handled. It is only used if check_sat[0] = True. Can be either equal to [True,'saturation'] or to a two elements list [True,XXX] where XXX is a float number. The float number gives the value above which it is assumed that a pixel is saturated. If check_sat[1]='saturation' then the value conf.saturation is taken.
	* mag_sat (float) = optional argument, default value is np.nan. Magnitude value returned by the function in case a star is saturated.
	* mag_zero (float) = optional argument, default value is np.nan. Magnitude value returned by the function in case a star has an intensity below the sky background.
	* error_messages (integer) = optional argument. It defines the level of information displayed in the console window about photometry errors. Default value is 1, can be equal to 0 (no information displayed), 1 (basic information) or 2 (more detailed information).
	---------------------
	Outputs
	* mag_stars (2D Numpy array) = returned measured magnitudes. Has the form [[m1_1, m1_2, ..., m1_P], [m2_1, m2_2, ..., m2_P], ..., [mN_1, MN_2, ..., MN_P]]. P=len(sizes) is the number of sets of photometry disks provided as input and N=len(coord) is the number of stars. 
	---------------------
	What can go wrong when computing magnitudes?
	* star coordinates are np.nan --> magnitude is np.nan
	* star is too close to border --> magnitude is np.nan
	* star has an intensity level below the sky background --> magnitude is mag_zero
	* star is saturated --> magnitude is mag_sat
	"""

	#bug corrections 17-06-2019
	#(1) mag_sat instead of np.nan in case of saturation
	#(2) removed unnecessary test xnotmax>xrangemax before translating image2
	#(3) replaced 	xnot = max(0,x-int(sizes[k][0])-4) // xrange = min(image.shape[0],x+int(sizes[k][0]+5))
	#		with  	xnot = max(0,x-int(sizes[k][0])-5) // xrange = min(image.shape[0],x+int(sizes[k][0]+4))

	if check_sat[1]=='saturation':
		check_sat[1] = conf.saturation
		
	maxsize = int(np.ceil(max(sizes)[0]))		#maximum radius of the photometry disk for the star center
	#-------------------------
	if method == 'centroid':
		image2 = np.array(image)
	#-------------------------
	
	error_image_border = []
	error_negative_intensity = []
		
	#loop on stars / asteroids
	mag_stars = []

	for j, xy in enumerate(coord):

		mag_methods = []

		if np.isnan(xy[0]):
			for k in range(len(sizes)):		
				mag_methods.append(np.nan)
		else:
			#coordinates for object center
			x = int(np.round(xy[0]))
			y = int(np.round(xy[1]))
			dx = xy[0] - x
			dy = xy[1] - y
		
			#-------------------------		
			saturation = False
			if check_sat[0]:
				xnot = max(0,x-maxsize-2)
				xrange = min(image.shape[0],x+maxsize+1)
				ynot = max(0,y-maxsize-2)
				yrange = min(image.shape[1],y+maxsize+1)
				try:
					maxsignal = np.max(image[xnot:xrange,ynot:yrange].flatten())
					
					if maxsignal>=check_sat[1]:
						saturation = True
				except:	#if image[xnot:xrange,ynot:yrange]=[] because point outside image
					pass
			#-------------------------		

			if saturation:
				#loop on apertures for photometry
				for k in range(len(sizes)):		
					mag_methods.append(mag_sat)
			else:
				#-------------------------
				if method == 'centroid':
					xnotmax = max(0,x-maxsize-4)
					xrangemax = min(image.shape[0],x+maxsize+3)
					ynotmax = max(0,y-maxsize-4)
					yrangemax = min(image.shape[1],y+maxsize+3)

					image2[xnotmax:xrangemax,ynotmax:yrangemax] = astro.translate_Bspline(image2[xnotmax:xrangemax,ynotmax:yrangemax],(-dx,-dy),3)
				#-------------------------
							
				#loop on apertures for photometry
				for k in range(len(sizes)):		

					#-------------------------
					if method == 'centroid':
						test = (x-sizes[k][0]-4<0)or(x+sizes[k][0]+3>image.shape[0])or(y-sizes[k][0]-4<0)or(y+sizes[k][0]+3>image.shape[1])
					else:
						test = (x-sizes[k][0]-1<0)or(x+sizes[k][0]>image.shape[0])or(y-sizes[k][0]-1<0)or(y+sizes[k][0]>image.shape[1])
					#-------------------------

					if 	test:
						error_image_border.append([j+1,k+1])
						mag_methods.append(np.nan)
					else:									#once this test is passed it is certain the star is within the image boundaries
						#selecting subimage
						xnot = max(0,x-int(sizes[k][2])-1)
						xrange = min(image.shape[0],x+int(sizes[k][2]))
						ynot = max(0,y-int(sizes[k][2])-1)
						yrange = min(image.shape[1],y+int(sizes[k][2]))

						subimage = image[xnot:xrange,ynot:yrange]
					
						#aperture photometry
						grid = np.indices(subimage.shape)
						radius = (grid[0]-x+xnot)**2+(grid[1]-y+ynot)**2

						cond1 = radius<=sizes[k][0]**2
						cond2 = (radius>=sizes[k][1]**2)*(radius<=sizes[k][2]**2)

						#-------------------------
						if method == 'centroid':
							subimage2 = image2[xnot:xrange,ynot:yrange]
							center = np.extract(cond1,subimage2)
						else:
							center = np.extract(cond1,subimage)
						#-------------------------
						crown = np.extract(cond2,subimage)			#the crown is purposely not shifted if method='centroid'

						#print(np.where(radius==np.min(np.min(radius))))
						#print(np.where(subimage==np.max(np.max(subimage))))
						#if (sys.version[0]=="2"):
						#	raw_input("ok?")
						#elif (sys.version[0]=="3"):
						#	input("ok?")

						#------ magnitude calculation ------
						floor = astro.get_median(np.tile(np.array(crown,dtype=np.float),(1,1,1)))[0]
						intensity = np.sum(np.array(center,dtype=np.float)-floor)
						if intensity <= 0.:
							error_negative_intensity.append([j+1,k+1])
							mag_methods.append(mag_zero)
						else:
							mag_methods.append(-2.5*np.log10(intensity))

				#-------------------------
				if method == 'centroid':
					xnotmax = max(0,x-maxsize-4)
					xrangemax = min(image.shape[0],x+maxsize+3)
					ynotmax = max(0,y-maxsize-4)
					yrangemax = min(image.shape[1],y+maxsize+3)
					image2[xnotmax:xrangemax,ynotmax:yrangemax] = image[xnotmax:xrangemax,ynotmax:yrangemax]
				#-------------------------
		mag_stars.append(mag_methods)
	
	if error_messages==1:
		print("")
		print("aspylib.photometry.aperture_photometry():")
		if len(error_image_border)>0:
			print('photometry.aperture_photometry():',len(error_image_border),'/',len(coord)*len(sizes),'measurement(s) on image border')
		if len(error_negative_intensity)>0:
			print('photometry.aperture_photometry():',len(error_negative_intensity),'/',len(coord)*len(sizes),'measurement(s) with <0 intensity')

	elif error_messages==2:
		print("")
		print("aspylib.photometry.aperture_photometry():")
		if len(error_image_border)>0:
			print('photometry.aperture_photometry():',len(error_image_border),'/',len(coord)*len(sizes),'measurement(s) on image border:')
			print(str(error_image_border)[1:-1])
		if len(error_negative_intensity)>0:
			print('photometry.aperture_photometry():',len(error_negative_intensity),'/',len(coord)*len(sizes),'measurement(s) with <0 intensity:')
			print(str(error_negative_intensity)[1:-1])

	#-------------------------
	if method == 'centroid':
		del image2
	#-------------------------
	
	mag_stars = np.array(mag_stars)
	return mag_stars

